import sys
import threading
import queue
import random
import struct
from typing import List

from jacdac.bus import Transport

HF2_CMD_INFO = 0x0002
HF2_CMD_DMESG = 0x0010

HF2_FLAG_SERIAL_OUT = 0x80
HF2_FLAG_SERIAL_ERR = 0xc0
HF2_FLAG_CMDPKT_LAST = 0x40
HF2_FLAG_CMDPKT_BODY = 0x00
HF2_FLAG_MASK = 0xc0
HF2_SIZE_MASK = 63

HF2_STATUS_OK = 0x00
HF2_STATUS_INVALID_CMD = 0x01
HF2_STATUS_EXEC_ERR = 0x02
HF2_STATUS_EVENT = 0x80

# the eventId is overlayed on the tag+status; the mask corresponds
# to the HF2_STATUS_EVENT above
HF2_EV_MASK = 0x800000

HF2_CMD_JDS_CONFIG = 0x0020
HF2_CMD_JDS_SEND = 0x0021
HF2_EV_JDS_PACKET = 0x800020


class HF2Error(Exception):
    pass


class HF2Transport(Transport):
    def _write(self, buf: bytes):
        frame = bytearray(64)
        pos = 0
        while True:
            l = len(buf) - pos
            if l <= 0:
                break
            if l > 63:
                l = 63
                frame[0] = HF2_FLAG_CMDPKT_BODY
            else:
                frame[0] = HF2_FLAG_CMDPKT_LAST
            frame[0] |= l
            frame[1:1+l] = buf[pos:pos+l]
            self.serial.write(frame)  # type: ignore
            pos += l

    def _on_serial(self, buf: bytes, is_error: bool):
        self.log("serial: %s" % buf.decode("utf-8"))

    def _on_jd_pkt(self, buf: bytes):
        # self.log("jd: " + buf.hex())
        if self.on_receive:
            self.on_receive(buf)

    def _on_event(self, buf: bytes):
        (evid,) = struct.unpack("<I", buf[0:4])
        if evid == HF2_EV_JDS_PACKET:
            self._on_jd_pkt(buf[4:])
        else:
            self.log("unknown event: 0x%x" % evid)

    def _read_loop(self):
        frames: List[bytes] = []
        while True:
            buf: bytes = self.serial.read(64)  # type: ignore
            tp = buf[0] & HF2_FLAG_MASK
            l = buf[0] & 63
            frame = buf[1:1+l]
            if tp & HF2_FLAG_SERIAL_OUT:
                self._on_serial(frame, tp == HF2_FLAG_SERIAL_ERR)
                continue
            frames.append(frame)
            if tp == HF2_FLAG_CMDPKT_BODY:
                pass
            else:
                assert tp == HF2_FLAG_CMDPKT_LAST
                r = b''.join(frames)
                frames = []
                if r[2] & HF2_STATUS_EVENT:
                    self._on_event(r)
                else:
                    self._msgs.put(r)

    def log(self, msg: str):
        print("HF2: %s" % msg)

    def _error(self, msg: str):
        self.log("error: %s" % msg)
        raise HF2Error("HF2: %s" % msg)

    def _talk(self, cmd: int, data: bytes = b'') -> bytes:
        with self._talk_lock:
            self._cmd_seq = (self._cmd_seq + 1) & 0xffff
            seq = self._cmd_seq
            payload = struct.pack("<IHH", cmd, seq, 0) + data
            self._write(payload)
            for repeat in range(3):
                try:
                    resp = self._msgs.get(block=True, timeout=1)
                except queue.Empty:
                    self._error("timeout for 0x%d" % cmd)
                (seq2, status, info) = struct.unpack("<HBB", resp[0:4])
                if seq != seq2:
                    self.log("packet out of sync (exp: %d, got: %d)" %
                             (seq, seq2))
                elif status == 0:
                    return resp[4:]
                else:
                    self._error("bad status: %d (info=%d) for 0x%x" %
                                (status, info, cmd))
            self._error("desync for 0x%x" % cmd)

    def _connect(self):
        info = self._talk(HF2_CMD_INFO)
        self.log("connected to '%s'" % info.decode("utf8"))
        self._talk(HF2_CMD_JDS_CONFIG, struct.pack("<I", 1))

    def send(self, pkt: bytes):
        self._talk(HF2_CMD_JDS_SEND, pkt)

    def __init__(self, portname: str) -> None:
        import serial # pyright: ignore
        self.serial: serial.Serial = serial.Serial(portname, 4_000_000)
        self._msgs: queue.Queue[bytes] = queue.Queue()
        self._cmd_seq = random.randint(0x1000, 0xffff)
        self._talk_lock = threading.Lock()
        self._reader_thread = threading.Thread(target=self._read_loop)
        self._reader_thread.daemon = True
        self._reader_thread.start()
        self._connect()


if __name__ == "__main__":
    hf2 = HF2Transport(sys.argv[1])



# Autogenerated constants for Rain gauge service
from jacdac.constants import *
from jacdac.system.constants import *
JD_SERVICE_CLASS_RAIN_GAUGE = const(0x13734c95)
JD_RAIN_GAUGE_REG_PRECIPITATION = const(JD_REG_READING)
JD_RAIN_GAUGE_REG_PRECIPITATION_PRECISION = const(JD_REG_READING_RESOLUTION)
JD_RAIN_GAUGE_PACK_FORMATS = {
    JD_RAIN_GAUGE_REG_PRECIPITATION: "u16.16",
    JD_RAIN_GAUGE_REG_PRECIPITATION_PRECISION: "u16.16"
}



# Autogenerated constants for Compass service
from jacdac.constants import *
from jacdac.system.constants import *
JD_SERVICE_CLASS_COMPASS = const(0x15b7b9bf)
JD_COMPASS_REG_HEADING = const(JD_REG_READING)
JD_COMPASS_REG_ENABLED = const(JD_REG_INTENSITY)
JD_COMPASS_REG_HEADING_ERROR = const(JD_REG_READING_ERROR)
JD_COMPASS_CMD_CALIBRATE = const(JD_CMD_CALIBRATE)
JD_COMPASS_PACK_FORMATS = {
    JD_COMPASS_REG_HEADING: "u16.16",
    JD_COMPASS_REG_ENABLED: "u8",
    JD_COMPASS_REG_HEADING_ERROR: "u16.16"
}



# Autogenerated file. Do not edit.
from jacdac.bus import Bus, SensorClient
from .constants import *
from typing import Optional


class LightLevelClient(SensorClient):
    """
    A sensor that measures luminosity level.
    Implements a client for the `Light level <https://microsoft.github.io/jacdac-docs/services/lightlevel>`_ service.

    """

    def __init__(self, bus: Bus, role: str, *, missing_light_level_value: float = None) -> None:
        super().__init__(bus, JD_SERVICE_CLASS_LIGHT_LEVEL, JD_LIGHT_LEVEL_PACK_FORMATS, role)
        self.missing_light_level_value = missing_light_level_value

    @property
    def light_level(self) -> Optional[float]:
        """
        Detect light level, _: /
        """
        self.refresh_reading()
        return self.register(JD_LIGHT_LEVEL_REG_LIGHT_LEVEL).float_value(self.missing_light_level_value, 100)

    @property
    def light_level_error(self) -> Optional[float]:
        """
        (Optional) Absolute estimated error of the reading value, _: /
        """
        return self.register(JD_LIGHT_LEVEL_REG_LIGHT_LEVEL_ERROR).float_value(100)

    @property
    def variant(self) -> Optional[LightLevelVariant]:
        """
        (Optional) The type of physical sensor., 
        """
        return self.register(JD_LIGHT_LEVEL_REG_VARIANT).value()

    



# Autogenerated constants for Weight Scale service
from enum import IntEnum
from jacdac.constants import *
from jacdac.system.constants import *
JD_SERVICE_CLASS_WEIGHT_SCALE = const(0x1f4d5040)


class WeightScaleVariant(IntEnum):
    BODY = const(0x1)
    FOOD = const(0x2)
    JEWELRY = const(0x3)


JD_WEIGHT_SCALE_REG_WEIGHT = const(JD_REG_READING)
JD_WEIGHT_SCALE_REG_WEIGHT_ERROR = const(JD_REG_READING_ERROR)
JD_WEIGHT_SCALE_REG_ZERO_OFFSET = const(0x80)
JD_WEIGHT_SCALE_REG_GAIN = const(0x81)
JD_WEIGHT_SCALE_REG_MAX_WEIGHT = const(JD_REG_MAX_READING)
JD_WEIGHT_SCALE_REG_MIN_WEIGHT = const(JD_REG_MIN_READING)
JD_WEIGHT_SCALE_REG_WEIGHT_RESOLUTION = const(JD_REG_READING_RESOLUTION)
JD_WEIGHT_SCALE_REG_VARIANT = const(JD_REG_VARIANT)
JD_WEIGHT_SCALE_CMD_CALIBRATE_ZERO_OFFSET = const(0x80)
JD_WEIGHT_SCALE_CMD_CALIBRATE_GAIN = const(0x81)
JD_WEIGHT_SCALE_PACK_FORMATS = {
    JD_WEIGHT_SCALE_REG_WEIGHT: "u16.16",
    JD_WEIGHT_SCALE_REG_WEIGHT_ERROR: "u16.16",
    JD_WEIGHT_SCALE_REG_ZERO_OFFSET: "u16.16",
    JD_WEIGHT_SCALE_REG_GAIN: "u16.16",
    JD_WEIGHT_SCALE_REG_MAX_WEIGHT: "u16.16",
    JD_WEIGHT_SCALE_REG_MIN_WEIGHT: "u16.16",
    JD_WEIGHT_SCALE_REG_WEIGHT_RESOLUTION: "u16.16",
    JD_WEIGHT_SCALE_REG_VARIANT: "u8",
    JD_WEIGHT_SCALE_CMD_CALIBRATE_GAIN: "u22.10"
}



# Autogenerated file. Do not edit.
from jacdac.bus import Bus, SensorClient
from .constants import *
from typing import Optional


class ECO2Client(SensorClient):
    """
    Measures equivalent CO₂ levels.
    Implements a client for the `Equivalent CO₂ <https://microsoft.github.io/jacdac-docs/services/eco2>`_ service.

    """

    def __init__(self, bus: Bus, role: str, *, missing_e_CO2_value: float = None) -> None:
        super().__init__(bus, JD_SERVICE_CLASS_E_CO2, JD_E_CO2_PACK_FORMATS, role, preferred_interval = 1000)
        self.missing_e_CO2_value = missing_e_CO2_value

    @property
    def e_CO2(self) -> Optional[float]:
        """
        Equivalent CO₂ (eCO₂) readings., _: ppm
        """
        self.refresh_reading()
        return self.register(JD_E_CO2_REG_E_CO2).value(self.missing_e_CO2_value)

    @property
    def e_CO2_error(self) -> Optional[float]:
        """
        (Optional) Error on the reading value., _: ppm
        """
        return self.register(JD_E_CO2_REG_E_CO2_ERROR).value()

    @property
    def min_e_CO2(self) -> Optional[float]:
        """
        Minimum measurable value, _: ppm
        """
        return self.register(JD_E_CO2_REG_MIN_E_CO2).value()

    @property
    def max_e_CO2(self) -> Optional[float]:
        """
        Minimum measurable value, _: ppm
        """
        return self.register(JD_E_CO2_REG_MAX_E_CO2).value()

    @property
    def variant(self) -> Optional[ECO2Variant]:
        """
        (Optional) Type of physical sensor and capabilities., 
        """
        return self.register(JD_E_CO2_REG_VARIANT).value()

    



# Autogenerated constants for Flex service
from jacdac.constants import *
from jacdac.system.constants import *
JD_SERVICE_CLASS_FLEX = const(0x1f47c6c6)
JD_FLEX_REG_BENDING = const(JD_REG_READING)
JD_FLEX_REG_LENGTH = const(0x180)
JD_FLEX_PACK_FORMATS = {
    JD_FLEX_REG_BENDING: "i1.15",
    JD_FLEX_REG_LENGTH: "u16"
}



# Autogenerated constants for Pressure Button service
from jacdac.constants import *
from jacdac.system.constants import *
JD_SERVICE_CLASS_PRESSURE_BUTTON = const(0x281740c3)
JD_PRESSURE_BUTTON_REG_THRESHOLD = const(JD_REG_ACTIVE_THRESHOLD)
JD_PRESSURE_BUTTON_PACK_FORMATS = {
    JD_PRESSURE_BUTTON_REG_THRESHOLD: "u0.16"
}



def const(v: int): return v
# A shim for the micropython const keyword

JD_VERSION = "0.2.5"

JD_SERIAL_HEADER_SIZE = const(16)
JD_SERIAL_MAX_PAYLOAD_SIZE = const(236)
JD_SERVICE_INDEX_MASK = const(0x3f)
JD_SERVICE_INDEX_INV_MASK = const(0xc0)
JD_SERVICE_INDEX_CRC_ACK = const(0x3f)
JD_SERVICE_INDEX_PIPE = const(0x3e)
JD_SERVICE_INDEX_CTRL = const(0x00)

JD_FRAME_FLAG_COMMAND = const(0x01)
JD_FRAME_FLAG_ACK_REQUESTED = const(0x02)
JD_FRAME_FLAG_IDENTIFIER_IS_SERVICE_CLASS = const(0x04)

JD_REGISTER_POLL_STREAMING_INTERVAL = 5000
JD_REGISTER_POLL_FIRST_REPORT_INTERVAL = 400
JD_REGISTER_POLL_REPORT_INTERVAL = 5001
JD_REGISTER_POLL_REPORT_MAX_INTERVAL = 60000
JD_REGISTER_POLL_REPORT_VOLATILE_INTERVAL = 1000
JD_REGISTER_POLL_REPORT_VOLATILE_MAX_INTERVAL = 5000
JD_REGISTER_OPTIONAL_POLL_COUNT = 3
JD_STREAMING_DEFAULT_INTERVAL = 50

JD_LOGGER_LISTENER_TIMEOUT = 3000

# Registers 0x001-0x07f - r/w common to all services
# Registers 0x080-0x0ff - r/w defined per-service
# Registers 0x100-0x17f - r/o common to all services
# Registers 0x180-0x1ff - r/o defined per-service
# Registers 0x200-0xeff - custom, defined per-service
# Registers 0xf00-0xfff - reserved for implementation, should not be on the wire

CMD_GET_REG = const(0x1000)
CMD_SET_REG = const(0x2000)
CMD_TYPE_MASK = const(0xf000)
CMD_REG_MASK = const(0x0fff)
CMD_EVENT_MASK = const(0x8000)
CMD_EVENT_CODE_MASK = const(0xff)
CMD_EVENT_COUNTER_MASK = const(0x7f)
CMD_EVENT_COUNTER_POS = const(8)

PIPE_PORT_SHIFT = const(7)
PIPE_COUNTER_MASK = const(0x001f)
PIPE_CLOSE_MASK = const(0x0020)
PIPE_METADATA_MASK = (0x0040)

DEVTOOLS_URL = "http://localhost:8081"
DEVTOOLS_SOCKET_URL = "ws://localhost:8081"

def JD_GET(code: int):
    assert 0 <= code and code <= 0xfff
    return CMD_GET_REG | code


def JD_SET(code: int):
    assert 0 <= code and code <= 0xfff
    return CMD_SET_REG | code



# Autogenerated constants for Gyroscope service
from jacdac.constants import *
from jacdac.system.constants import *
JD_SERVICE_CLASS_GYROSCOPE = const(0x1e1b06f2)
JD_GYROSCOPE_REG_ROTATION_RATES = const(JD_REG_READING)
JD_GYROSCOPE_REG_ROTATION_RATES_ERROR = const(JD_REG_READING_ERROR)
JD_GYROSCOPE_REG_MAX_RATE = const(JD_REG_READING_RANGE)
JD_GYROSCOPE_REG_MAX_RATES_SUPPORTED = const(JD_REG_SUPPORTED_RANGES)
JD_GYROSCOPE_PACK_FORMATS = {
    JD_GYROSCOPE_REG_ROTATION_RATES: "i12.20 i12.20 i12.20",
    JD_GYROSCOPE_REG_ROTATION_RATES_ERROR: "u12.20",
    JD_GYROSCOPE_REG_MAX_RATE: "u12.20",
    JD_GYROSCOPE_REG_MAX_RATES_SUPPORTED: "r: u12.20"
}



# Autogenerated constants for Sound Spectrum service
from jacdac.constants import *
from jacdac.system.constants import *
JD_SERVICE_CLASS_SOUND_SPECTRUM = const(0x157abc1e)
JD_SOUND_SPECTRUM_REG_FREQUENCY_BINS = const(JD_REG_READING)
JD_SOUND_SPECTRUM_REG_ENABLED = const(JD_REG_INTENSITY)
JD_SOUND_SPECTRUM_REG_FFT_POW2_SIZE = const(0x80)
JD_SOUND_SPECTRUM_REG_MIN_DECIBELS = const(0x81)
JD_SOUND_SPECTRUM_REG_MAX_DECIBELS = const(0x82)
JD_SOUND_SPECTRUM_REG_SMOOTHING_TIME_CONSTANT = const(0x83)
JD_SOUND_SPECTRUM_PACK_FORMATS = {
    JD_SOUND_SPECTRUM_REG_FREQUENCY_BINS: "b",
    JD_SOUND_SPECTRUM_REG_ENABLED: "u8",
    JD_SOUND_SPECTRUM_REG_FFT_POW2_SIZE: "u8",
    JD_SOUND_SPECTRUM_REG_MIN_DECIBELS: "i16",
    JD_SOUND_SPECTRUM_REG_MAX_DECIBELS: "i16",
    JD_SOUND_SPECTRUM_REG_SMOOTHING_TIME_CONSTANT: "u0.8"
}



# Autogenerated constants for Accelerometer service
from jacdac.constants import *
from jacdac.system.constants import *
JD_SERVICE_CLASS_ACCELEROMETER = const(0x1f140409)
JD_ACCELEROMETER_REG_FORCES = const(JD_REG_READING)
JD_ACCELEROMETER_REG_FORCES_ERROR = const(JD_REG_READING_ERROR)
JD_ACCELEROMETER_REG_MAX_FORCE = const(JD_REG_READING_RANGE)
JD_ACCELEROMETER_REG_MAX_FORCES_SUPPORTED = const(JD_REG_SUPPORTED_RANGES)
JD_ACCELEROMETER_EV_TILT_UP = const(0x81)
JD_ACCELEROMETER_EV_TILT_DOWN = const(0x82)
JD_ACCELEROMETER_EV_TILT_LEFT = const(0x83)
JD_ACCELEROMETER_EV_TILT_RIGHT = const(0x84)
JD_ACCELEROMETER_EV_FACE_UP = const(0x85)
JD_ACCELEROMETER_EV_FACE_DOWN = const(0x86)
JD_ACCELEROMETER_EV_FREEFALL = const(0x87)
JD_ACCELEROMETER_EV_SHAKE = const(0x8b)
JD_ACCELEROMETER_EV_FORCE_2G = const(0x8c)
JD_ACCELEROMETER_EV_FORCE_3G = const(0x88)
JD_ACCELEROMETER_EV_FORCE_6G = const(0x89)
JD_ACCELEROMETER_EV_FORCE_8G = const(0x8a)
JD_ACCELEROMETER_PACK_FORMATS = {
    JD_ACCELEROMETER_REG_FORCES: "i12.20 i12.20 i12.20",
    JD_ACCELEROMETER_REG_FORCES_ERROR: "u12.20",
    JD_ACCELEROMETER_REG_MAX_FORCE: "u12.20",
    JD_ACCELEROMETER_REG_MAX_FORCES_SUPPORTED: "r: u12.20"
}



# Autogenerated constants for Power service
from enum import IntEnum
from jacdac.constants import *
from jacdac.system.constants import *
JD_SERVICE_CLASS_POWER = const(0x1fa4c95a)


class PowerPowerStatus(IntEnum):
    DISALLOWED = const(0x0)
    POWERING = const(0x1)
    OVERLOAD = const(0x2)
    OVERPROVISION = const(0x3)


JD_POWER_REG_ALLOWED = const(JD_REG_INTENSITY)
JD_POWER_REG_MAX_POWER = const(JD_REG_MAX_POWER)
JD_POWER_REG_POWER_STATUS = const(0x181)
JD_POWER_REG_CURRENT_DRAW = const(JD_REG_READING)
JD_POWER_REG_BATTERY_VOLTAGE = const(0x180)
JD_POWER_REG_BATTERY_CHARGE = const(0x182)
JD_POWER_REG_BATTERY_CAPACITY = const(0x183)
JD_POWER_REG_KEEP_ON_PULSE_DURATION = const(0x80)
JD_POWER_REG_KEEP_ON_PULSE_PERIOD = const(0x81)
JD_POWER_CMD_SHUTDOWN = const(0x80)
JD_POWER_EV_POWER_STATUS_CHANGED = const(JD_EV_CHANGE)
JD_POWER_PACK_FORMATS = {
    JD_POWER_REG_ALLOWED: "u8",
    JD_POWER_REG_MAX_POWER: "u16",
    JD_POWER_REG_POWER_STATUS: "u8",
    JD_POWER_REG_CURRENT_DRAW: "u16",
    JD_POWER_REG_BATTERY_VOLTAGE: "u16",
    JD_POWER_REG_BATTERY_CHARGE: "u0.16",
    JD_POWER_REG_BATTERY_CAPACITY: "u32",
    JD_POWER_REG_KEEP_ON_PULSE_DURATION: "u16",
    JD_POWER_REG_KEEP_ON_PULSE_PERIOD: "u16",
    JD_POWER_EV_POWER_STATUS_CHANGED: "u8"
}



# Autogenerated constants for DMX service
from jacdac.constants import *
from jacdac.system.constants import *
JD_SERVICE_CLASS_DMX = const(0x11cf8c05)
JD_DMX_REG_ENABLED = const(JD_REG_INTENSITY)
JD_DMX_CMD_SEND = const(0x80)
JD_DMX_PACK_FORMATS = {
    JD_DMX_REG_ENABLED: "u8",
    JD_DMX_CMD_SEND: "b"
}



# Autogenerated constants for Motion service
from enum import IntEnum
from jacdac.constants import *
from jacdac.system.constants import *
JD_SERVICE_CLASS_MOTION = const(0x1179a749)


class MotionVariant(IntEnum):
    PIR = const(0x1)


JD_MOTION_REG_MOVING = const(JD_REG_READING)
JD_MOTION_REG_MAX_DISTANCE = const(0x180)
JD_MOTION_REG_ANGLE = const(0x181)
JD_MOTION_REG_VARIANT = const(JD_REG_VARIANT)
JD_MOTION_EV_MOVEMENT = const(JD_EV_ACTIVE)
JD_MOTION_PACK_FORMATS = {
    JD_MOTION_REG_MOVING: "u8",
    JD_MOTION_REG_MAX_DISTANCE: "u16.16",
    JD_MOTION_REG_ANGLE: "u16",
    JD_MOTION_REG_VARIANT: "u8"
}



# Autogenerated constants for Indexed screen service
from jacdac.constants import *
from jacdac.system.constants import *
JD_SERVICE_CLASS_INDEXED_SCREEN = const(0x16fa36e5)
JD_INDEXED_SCREEN_CMD_START_UPDATE = const(0x81)
JD_INDEXED_SCREEN_CMD_SET_PIXELS = const(0x83)
JD_INDEXED_SCREEN_REG_BRIGHTNESS = const(JD_REG_INTENSITY)
JD_INDEXED_SCREEN_REG_PALETTE = const(0x80)
JD_INDEXED_SCREEN_REG_BITS_PER_PIXEL = const(0x180)
JD_INDEXED_SCREEN_REG_WIDTH = const(0x181)
JD_INDEXED_SCREEN_REG_HEIGHT = const(0x182)
JD_INDEXED_SCREEN_REG_WIDTH_MAJOR = const(0x81)
JD_INDEXED_SCREEN_REG_UP_SAMPLING = const(0x82)
JD_INDEXED_SCREEN_REG_ROTATION = const(0x83)
JD_INDEXED_SCREEN_PACK_FORMATS = {
    JD_INDEXED_SCREEN_CMD_START_UPDATE: "u16 u16 u16 u16",
    JD_INDEXED_SCREEN_CMD_SET_PIXELS: "b",
    JD_INDEXED_SCREEN_REG_BRIGHTNESS: "u0.8",
    JD_INDEXED_SCREEN_REG_PALETTE: "r: u8 u8 u8 u8",
    JD_INDEXED_SCREEN_REG_BITS_PER_PIXEL: "u8",
    JD_INDEXED_SCREEN_REG_WIDTH: "u16",
    JD_INDEXED_SCREEN_REG_HEIGHT: "u16",
    JD_INDEXED_SCREEN_REG_WIDTH_MAJOR: "u8",
    JD_INDEXED_SCREEN_REG_UP_SAMPLING: "u8",
    JD_INDEXED_SCREEN_REG_ROTATION: "u16"
}



# Autogenerated constants for Character Screen service
from enum import IntEnum
from jacdac.constants import *
from jacdac.system.constants import *
JD_SERVICE_CLASS_CHARACTER_SCREEN = const(0x1f37c56a)


class CharacterScreenVariant(IntEnum):
    LCD = const(0x1)
    OLED = const(0x2)
    BRAILLE = const(0x3)


class CharacterScreenTextDirection(IntEnum):
    LEFT_TO_RIGHT = const(0x1)
    RIGHT_TO_LEFT = const(0x2)


JD_CHARACTER_SCREEN_REG_MESSAGE = const(JD_REG_VALUE)
JD_CHARACTER_SCREEN_REG_BRIGHTNESS = const(JD_REG_INTENSITY)
JD_CHARACTER_SCREEN_REG_VARIANT = const(JD_REG_VARIANT)
JD_CHARACTER_SCREEN_REG_TEXT_DIRECTION = const(0x82)
JD_CHARACTER_SCREEN_REG_ROWS = const(0x180)
JD_CHARACTER_SCREEN_REG_COLUMNS = const(0x181)
JD_CHARACTER_SCREEN_PACK_FORMATS = {
    JD_CHARACTER_SCREEN_REG_MESSAGE: "s",
    JD_CHARACTER_SCREEN_REG_BRIGHTNESS: "u0.16",
    JD_CHARACTER_SCREEN_REG_VARIANT: "u8",
    JD_CHARACTER_SCREEN_REG_TEXT_DIRECTION: "u8",
    JD_CHARACTER_SCREEN_REG_ROWS: "u8",
    JD_CHARACTER_SCREEN_REG_COLUMNS: "u8"
}



# Autogenerated file. Do not edit.
from jacdac.bus import Bus, SensorClient
from .constants import *
from typing import Optional


class HeartRateClient(SensorClient):
    """
    A sensor approximating the heart rate. 
     * 
     * 
     * **Jacdac is NOT suitable for medical devices and should NOT be used in any kind of device to diagnose or treat any medical conditions.**
    Implements a client for the `Heart Rate <https://microsoft.github.io/jacdac-docs/services/heartrate>`_ service.

    """

    def __init__(self, bus: Bus, role: str, *, missing_heart_rate_value: float = None) -> None:
        super().__init__(bus, JD_SERVICE_CLASS_HEART_RATE, JD_HEART_RATE_PACK_FORMATS, role, preferred_interval = 1000)
        self.missing_heart_rate_value = missing_heart_rate_value

    @property
    def heart_rate(self) -> Optional[float]:
        """
        The estimated heart rate., _: bpm
        """
        self.refresh_reading()
        return self.register(JD_HEART_RATE_REG_HEART_RATE).value(self.missing_heart_rate_value)

    @property
    def heart_rate_error(self) -> Optional[float]:
        """
        (Optional) The estimated error on the reported sensor data., _: bpm
        """
        return self.register(JD_HEART_RATE_REG_HEART_RATE_ERROR).value()

    @property
    def variant(self) -> Optional[HeartRateVariant]:
        """
        (Optional) The type of physical sensor, 
        """
        return self.register(JD_HEART_RATE_REG_VARIANT).value()

    



# Autogenerated constants for Random Number Generator service
from enum import IntEnum
from jacdac.constants import *
from jacdac.system.constants import *
JD_SERVICE_CLASS_RNG = const(0x1789f0a2)


class RngVariant(IntEnum):
    QUANTUM = const(0x1)
    ADCNOISE = const(0x2)
    WEB_CRYPTO = const(0x3)


JD_RNG_REG_RANDOM = const(0x180)
JD_RNG_REG_VARIANT = const(JD_REG_VARIANT)
JD_RNG_PACK_FORMATS = {
    JD_RNG_REG_RANDOM: "b",
    JD_RNG_REG_VARIANT: "u8"
}



# Autogenerated constants for 7-segment display service
from jacdac.constants import *
from jacdac.system.constants import *
JD_SERVICE_CLASS_SEVEN_SEGMENT_DISPLAY = const(0x196158f7)
JD_SEVEN_SEGMENT_DISPLAY_REG_DIGITS = const(JD_REG_VALUE)
JD_SEVEN_SEGMENT_DISPLAY_REG_BRIGHTNESS = const(JD_REG_INTENSITY)
JD_SEVEN_SEGMENT_DISPLAY_REG_DOUBLE_DOTS = const(0x80)
JD_SEVEN_SEGMENT_DISPLAY_REG_DIGIT_COUNT = const(0x180)
JD_SEVEN_SEGMENT_DISPLAY_REG_DECIMAL_POINT = const(0x181)
JD_SEVEN_SEGMENT_DISPLAY_CMD_SET_NUMBER = const(0x80)
JD_SEVEN_SEGMENT_DISPLAY_PACK_FORMATS = {
    JD_SEVEN_SEGMENT_DISPLAY_REG_DIGITS: "b",
    JD_SEVEN_SEGMENT_DISPLAY_REG_BRIGHTNESS: "u0.16",
    JD_SEVEN_SEGMENT_DISPLAY_REG_DOUBLE_DOTS: "u8",
    JD_SEVEN_SEGMENT_DISPLAY_REG_DIGIT_COUNT: "u8",
    JD_SEVEN_SEGMENT_DISPLAY_REG_DECIMAL_POINT: "u8",
    JD_SEVEN_SEGMENT_DISPLAY_CMD_SET_NUMBER: "f64"
}



from jacdac.pack import jdpack
from ..bus import Bus, Server
from ..packet import JDPacket
from .constants import *


class ProtoTestServer(Server):
    def __init__(self, bus: Bus) -> None:
        super().__init__(bus, JD_SERVICE_CLASS_PROTO_TEST)
        self.rw_bool = 0
        self.rw_i32 = 0
        self.rw_u32 = 0
        self.rw_str = ""
        self.rw_bytes = bytearray(0)
        self.rw_u8_string = [0, ""]

    def handle_packet(self, pkt: JDPacket):
        cmd = pkt.service_command
        rw_bool = self.handle_reg(pkt, JD_PROTO_TEST_REG_RW_BOOL,
                                  JD_PROTO_TEST_PACK_FORMATS[JD_PROTO_TEST_REG_RW_BOOL], self.rw_bool)
        if rw_bool != self.rw_bool:
            self.rw_bool = rw_bool
            self.send_event(JD_PROTO_TEST_EV_E_BOOL, jdpack(
                JD_PROTO_TEST_PACK_FORMATS[JD_PROTO_TEST_REG_RW_BOOL], self.rw_bool))
        self.handle_reg(pkt, JD_PROTO_TEST_REG_RO_BOOL,
                        JD_PROTO_TEST_PACK_FORMATS[JD_PROTO_TEST_REG_RO_BOOL], self.rw_bool)
        if cmd == JD_PROTO_TEST_CMD_C_BOOL:
            [self.rw_bool] = pkt.unpack(JD_PROTO_TEST_PACK_FORMATS[JD_PROTO_TEST_CMD_C_BOOL])

        rw_i32 = self.handle_reg(
            pkt, JD_PROTO_TEST_REG_RW_I32, JD_PROTO_TEST_PACK_FORMATS[JD_PROTO_TEST_REG_RW_I32], self.rw_i32)
        if rw_i32 != self.rw_i32:
            self.rw_i32 = rw_i32
            self.send_event(JD_PROTO_TEST_EV_E_I32, jdpack(
                JD_PROTO_TEST_PACK_FORMATS[JD_PROTO_TEST_REG_RW_I32], self.rw_i32))
        self.handle_reg(pkt, JD_PROTO_TEST_REG_RO_I32,
                        JD_PROTO_TEST_PACK_FORMATS[JD_PROTO_TEST_REG_RO_I32], self.rw_i32)
        if cmd == JD_PROTO_TEST_CMD_C_I32:
            [self.rw_i32] = pkt.unpack(JD_PROTO_TEST_PACK_FORMATS[JD_PROTO_TEST_CMD_C_I32])

        rw_u32 = self.handle_reg(
            pkt, JD_PROTO_TEST_REG_RW_U32, JD_PROTO_TEST_PACK_FORMATS[JD_PROTO_TEST_REG_RW_U32], self.rw_u32)
        if rw_u32 != self.rw_u32:
            self.rw_u32 = rw_u32
            self.send_event(JD_PROTO_TEST_EV_E_U32, jdpack(
                JD_PROTO_TEST_PACK_FORMATS[JD_PROTO_TEST_REG_RW_U32], self.rw_u32))
        self.handle_reg(pkt, JD_PROTO_TEST_REG_RO_U32,
                        JD_PROTO_TEST_PACK_FORMATS[JD_PROTO_TEST_REG_RO_U32], self.rw_u32)
        if cmd == JD_PROTO_TEST_CMD_C_U32:
            [self.rw_u32] = pkt.unpack(JD_PROTO_TEST_PACK_FORMATS[JD_PROTO_TEST_CMD_C_U32])

        rw_str = self.handle_reg(pkt, JD_PROTO_TEST_REG_RW_STRING,
                                 JD_PROTO_TEST_PACK_FORMATS[JD_PROTO_TEST_REG_RW_STRING], self.rw_str)
        if rw_str != self.rw_str:
            self.rw_str = rw_str
            self.send_event(JD_PROTO_TEST_EV_E_STRING, jdpack(
                JD_PROTO_TEST_PACK_FORMATS[JD_PROTO_TEST_REG_RW_STRING], self.rw_str))
        self.handle_reg(pkt, JD_PROTO_TEST_REG_RO_STRING,
                        JD_PROTO_TEST_PACK_FORMATS[JD_PROTO_TEST_REG_RO_STRING], self.rw_str)
        if cmd == JD_PROTO_TEST_CMD_C_STRING:
            [self.rw_str] = pkt.unpack(JD_PROTO_TEST_PACK_FORMATS[JD_PROTO_TEST_CMD_C_STRING])

        rw_bytes = self.handle_reg(pkt, JD_PROTO_TEST_REG_RW_BYTES,
                                 JD_PROTO_TEST_PACK_FORMATS[JD_PROTO_TEST_REG_RW_BYTES], self.rw_bytes)
        if rw_bytes != self.rw_bytes:
            self.rw_bytes = rw_bytes
            self.send_event(JD_PROTO_TEST_EV_E_BYTES, jdpack(
                JD_PROTO_TEST_PACK_FORMATS[JD_PROTO_TEST_REG_RW_BYTES], self.rw_bytes))
        self.handle_reg(pkt, JD_PROTO_TEST_REG_RO_BYTES,
                        JD_PROTO_TEST_PACK_FORMATS[JD_PROTO_TEST_REG_RO_BYTES], self.rw_bytes)
        if cmd == JD_PROTO_TEST_CMD_C_BYTES:
            [self.rw_bytes] = pkt.unpack(JD_PROTO_TEST_PACK_FORMATS[JD_PROTO_TEST_CMD_C_BYTES])

        return super().handle_packet(pkt)


"""    
    export class ProtocolTestServer extends JDServiceServer {
    private rwBytes: JDRegisterServer<[Uint8Array]>

    constructor() {
        super(SRV_PROTO_TEST)

        this.init<[boolean]>(
            ProtoTestReg.RwBool,
            ProtoTestReg.RoBool,
            ProtoTestCmd.CBool,
            ProtoTestEvent.EBool,
            false
        )
        this.init<[number]>(
            ProtoTestReg.RwI32,
            ProtoTestReg.RoI32,
            ProtoTestCmd.CI32,
            ProtoTestEvent.EI32,
            0
        )
        this.init<[number]>(
            ProtoTestReg.RwU32,
            ProtoTestReg.RoU32,
            ProtoTestCmd.CU32,
            ProtoTestEvent.EU32,
            0
        )
        this.init<[string]>(
            ProtoTestReg.RwString,
            ProtoTestReg.RoString,
            ProtoTestCmd.CString,
            ProtoTestEvent.EString,
            ""
        )
        this.rwBytes = this.init<[Uint8Array]>(
            ProtoTestReg.RwBytes,
            ProtoTestReg.RoBytes,
            ProtoTestCmd.CBytes,
            ProtoTestEvent.EBytes,
            new Uint8Array(0)
        )
        this.init<[number, number, number, number]>(
            ProtoTestReg.RwI8U8U16I32,
            ProtoTestReg.RoI8U8U16I32,
            ProtoTestCmd.CI8U8U16I32,
            ProtoTestEvent.EI8U8U16I32,
            0,
            0,
            0,
            0
        )
        this.init<[number, string]>(
            ProtoTestReg.RwU8String,
            ProtoTestReg.RoU8String,
            ProtoTestCmd.CU8String,
            ProtoTestEvent.EU8String,
            0,
            ""
        )

        this.addCommand(
            ProtoTestCmd.CReportPipe,
            this.handleReportPipe.bind(this)
        )
    }

    private init<TValues extends any[]>(
        rwi: number,
        roi: number,
        ci: number,
        ei: number,
        ...values: TValues
    ) {
        const rw = this.addRegister(rwi, values)
        const ro = this.addRegister(roi, rw.values())
        rw.on(CHANGE, () => {
            ro.setValues(rw.values())
            this.sendEvent(ei, rw.data)
        })
        this.addCommand(ci, pkt =>
            rw.setValues(jdunpack(pkt.data, rw.specification.packFormat))
        )
        return rw
    }

    private async handleReportPipe(pkt: Packet) {
        const pipe = OutPipe.from(this.device.bus, pkt, true)
        await pipe.respondForEach(this.rwBytes.data, (b: number) => {
            const buf = new Uint8Array(1)
            buf[0] = b
            return jdpack<[Uint8Array]>("b", [buf])
        })
    }
}
"""



# Autogenerated file. Do not edit.
from jacdac.bus import Bus, Client, EventHandlerFn, UnsubscribeFn
from .constants import *
from typing import Optional


class JacscriptManagerClient(Client):
    """
    Allows for deployment and control over Jacscript virtual machine.
     * 
     * Programs start automatically after device restart or uploading of new program.
     * You can stop programs until next reset by setting the `running` register to `0`.
     * 
     * TODO - debug interface:
     * * read-globals command/register
     * * globals-changed pipe
     * * breakpoint command
     * * some performance monitoring?
    Implements a client for the `Jacscript Manager <https://microsoft.github.io/jacdac-docs/services/jacscriptmanager>`_ service.

    """

    def __init__(self, bus: Bus, role: str) -> None:
        super().__init__(bus, JD_SERVICE_CLASS_JACSCRIPT_MANAGER, JD_JACSCRIPT_MANAGER_PACK_FORMATS, role)


    @property
    def running(self) -> Optional[bool]:
        """
        Indicates if the program is currently running.
        To restart the program, stop it (write `0`), read back the register to make sure it's stopped,
        start it, and read back., 
        """
        return self.register(JD_JACSCRIPT_MANAGER_REG_RUNNING).bool_value()

    @running.setter
    def running(self, value: bool) -> None:
        self.register(JD_JACSCRIPT_MANAGER_REG_RUNNING).set_values(value)


    @property
    def autostart(self) -> Optional[bool]:
        """
        Indicates wheather the program should be re-started upon `reboot()` or `panic()`.
        Defaults to `true`., 
        """
        return self.register(JD_JACSCRIPT_MANAGER_REG_AUTOSTART).bool_value()

    @autostart.setter
    def autostart(self, value: bool) -> None:
        self.register(JD_JACSCRIPT_MANAGER_REG_AUTOSTART).set_values(value)


    @property
    def logging(self) -> Optional[bool]:
        """
        `log_message` reports are only sent when this is `true`.
        It defaults to `false`., 
        """
        return self.register(JD_JACSCRIPT_MANAGER_REG_LOGGING).bool_value()

    @logging.setter
    def logging(self, value: bool) -> None:
        self.register(JD_JACSCRIPT_MANAGER_REG_LOGGING).set_values(value)


    @property
    def program_size(self) -> Optional[int]:
        """
        The size of current program., 
        """
        return self.register(JD_JACSCRIPT_MANAGER_REG_PROGRAM_SIZE).value()

    @property
    def program_hash(self) -> Optional[int]:
        """
        Return FNV1A hash of the current bytecode., 
        """
        return self.register(JD_JACSCRIPT_MANAGER_REG_PROGRAM_HASH).value()

    def on_program_panic(self, handler: EventHandlerFn) -> UnsubscribeFn:
        """
        Emitted when the program calls `panic(panic_code)` or `reboot()` (`panic_code == 0` in that case).
        The byte offset in byte code of the call is given in `program_counter`.
        The program will restart immediately when `panic_code == 0` or in a few seconds otherwise.
        """
        return self.on_event(JD_JACSCRIPT_MANAGER_EV_PROGRAM_PANIC, handler)

    def on_program_change(self, handler: EventHandlerFn) -> UnsubscribeFn:
        """
        Emitted after bytecode of the program has changed.
        """
        return self.on_event(JD_JACSCRIPT_MANAGER_EV_PROGRAM_CHANGE, handler)


    def deploy_bytecode(self, bytecode_size: int) -> None:
        """
        Open pipe for streaming in the bytecode of the program. The size of the bytecode has to be declared upfront.
        To clear the program, use `bytecode_size == 0`.
        The bytecode is streamed over regular pipe data packets.
        The bytecode shall be fully written into flash upon closing the pipe.
        If `autostart` is true, the program will start after being deployed.
        The data payloads, including the last one, should have a size that is a multiple of 32 bytes.
        Thus, the initial bytecode_size also needs to be a multiple of 32.
        """
        self.send_cmd_packed(JD_JACSCRIPT_MANAGER_CMD_DEPLOY_BYTECODE, bytecode_size)
    



from ..bus import Bus
from .server import RealTimeClockServer
from time import sleep, localtime

if __name__ == '__main__':
    bus = Bus()

    clock = RealTimeClockServer(bus)
    while True:
        print(localtime())
        sleep(5)



# Autogenerated file. Do not edit.
from jacdac.bus import Bus, SensorClient
from .constants import *
from typing import Optional, Tuple


class RoverClient(SensorClient):
    """
    A roving robot.
    Implements a client for the `Rover <https://microsoft.github.io/jacdac-docs/services/rover>`_ service.

    """

    def __init__(self, bus: Bus, role: str, *, missing_kinematics_value: Tuple[float, float, float, float, float] = None) -> None:
        super().__init__(bus, JD_SERVICE_CLASS_ROVER, JD_ROVER_PACK_FORMATS, role)
        self.missing_kinematics_value = missing_kinematics_value

    @property
    def kinematics(self) -> Optional[Tuple[float, float, float, float, float]]:
        """
        The current position and orientation of the robot., x: cm,y: cm,vx: cm/s,vy: cm/s,heading: °
        """
        self.refresh_reading()
        return self.register(JD_ROVER_REG_KINEMATICS).value(self.missing_kinematics_value)

    



# Autogenerated constants for Rotary encoder service
from jacdac.constants import *
from jacdac.system.constants import *
JD_SERVICE_CLASS_ROTARY_ENCODER = const(0x10fa29c9)
JD_ROTARY_ENCODER_REG_POSITION = const(JD_REG_READING)
JD_ROTARY_ENCODER_REG_CLICKS_PER_TURN = const(0x180)
JD_ROTARY_ENCODER_REG_CLICKER = const(0x181)
JD_ROTARY_ENCODER_PACK_FORMATS = {
    JD_ROTARY_ENCODER_REG_POSITION: "i32",
    JD_ROTARY_ENCODER_REG_CLICKS_PER_TURN: "u16",
    JD_ROTARY_ENCODER_REG_CLICKER: "u8"
}



# Autogenerated constants for Settings service
from jacdac.constants import *
from jacdac.system.constants import *
JD_SERVICE_CLASS_SETTINGS = const(0x1107dc4a)
JD_SETTINGS_CMD_GET = const(0x80)
JD_SETTINGS_CMD_SET = const(0x81)
JD_SETTINGS_CMD_DELETE = const(0x84)
JD_SETTINGS_CMD_LIST_KEYS = const(0x82)
JD_SETTINGS_CMD_LIST = const(0x83)
JD_SETTINGS_CMD_CLEAR = const(0x85)
JD_SETTINGS_EV_CHANGE = const(JD_EV_CHANGE)
JD_SETTINGS_PACK_FORMATS = {
    JD_SETTINGS_CMD_GET: "s",
    JD_SETTINGS_CMD_SET: "z b",
    JD_SETTINGS_CMD_DELETE: "s",
    JD_SETTINGS_CMD_LIST_KEYS: "b[12]",
    JD_SETTINGS_CMD_LIST: "b[12]"
}



from typing import Callable, Optional
from jacdac.constants import JD_GET
from ..bus import Bus, Server, rand_u64
from ..packet import JDPacket
from .constants import JD_SERVICE_CLASS_RNG, JD_RNG_REG_RANDOM, JD_RNG_REG_VARIANT, RngVariant

RngGenerator = Callable[..., bytearray]


class RngServer(Server):
    """
    Generates random numbers using the python.getrandbits function.
    Implements a client for the `Random Number Generator <https://microsoft.github.io/jacdac-docs/services/rng>`_ service.
    """

    def __init__(self, bus: Bus, *, variant: Optional[RngVariant] = None, generator: RngGenerator = rand_u64) -> None:
        """Constructs a new rng server

        Args:
            bus (Bus): Jacdac bus
            variant (Optional[RngVariant], optional): Describes the random generation process. Defaults to None.
            generator (RngGenerator, optional): Function that generates 64 random bytes. Defaults to rand_u64.
        """
        super().__init__(bus, JD_SERVICE_CLASS_RNG)
        self.variant = variant
        self.generator = generator

    def handle_packet(self, pkt: JDPacket):
        cmd = pkt.service_command
        if cmd == JD_GET(JD_RNG_REG_RANDOM):
            self._handle_get_random(pkt)
        elif cmd == JD_GET(JD_RNG_REG_VARIANT):
            self._handle_get_variant(pkt)
        else:
            super().handle_packet(pkt)

    def _handle_get_random(self, pkt: JDPacket):
        r = self.generator()
        self.debug("random {}", r.hex())
        self.send_report(JDPacket.packed(pkt.service_command, "b", r))

    def _handle_get_variant(self, pkt: JDPacket):
        if self.variant is None:
            self.send_report(pkt.not_implemented())
        else:
            self.send_report(JDPacket.packed(
                pkt.service_command, "u8", self.variant))



from typing import cast
from jacdac.bus import Bus, Server, JDPacket, OutPipe
from jacdac.pack import jdpack
from .constants import *
from ..settings_file import SettingsFile
from os import path


class SettingsServer(Server):
    """
    Non-volatile key-value storage interface for storing settings.
    Implements a server for the `Settings <https://microsoft.github.io/jacdac-docs/services/settings>`_ service.
    """

    def __init__(self, bus: Bus) -> None:
        super().__init__(bus, JD_SERVICE_CLASS_SETTINGS)
        file_name = path.join(self.bus.storage_dir, "settings.json")
        self.settings = SettingsFile(file_name)

    def handle_packet(self, pkt: JDPacket):
        cmd = pkt.service_command
        if cmd == JD_SETTINGS_CMD_CLEAR:
            self._handle_clear(pkt)
        elif cmd == JD_SETTINGS_CMD_LIST_KEYS:
            self._handle_list_keys(pkt)
        elif cmd == JD_SETTINGS_CMD_LIST:
            self._handle_list(pkt)
        elif cmd == JD_SETTINGS_CMD_GET:
            self._handle_get(pkt)
        elif cmd == JD_SETTINGS_CMD_SET:
            self._handle_set(pkt)
        elif cmd == JD_SETTINGS_CMD_DELETE:
            self._handle_delete(pkt)
        return super().handle_packet(pkt)

    def _handle_get(self, pkt: JDPacket):
        key: str = cast(str, pkt.unpack("s")[0])
        value = self.settings.read(key)
        self.send_report(JDPacket.packed(
            JD_SETTINGS_CMD_GET, "z b", key, value))

    def _handle_delete(self, pkt: JDPacket):
        [key] = pkt.unpack("s")
        self.debug("delete key {}", key)
        self.settings.delete(cast(str, key))
        self.send_change_event()

    def _handle_set(self, pkt: JDPacket):
        [key, value] = pkt.unpack("z b")
        self.debug("set key {}", key)
        self.settings.write(cast(str, key), cast(bytearray, value))
        self.send_change_event()

    def _handle_clear(self, pkt: JDPacket):
        self.debug("clear keys")
        self.settings.clear()
        self.send_change_event()

    def _handle_list(self, pkt: JDPacket):
        pipe = OutPipe(self.bus, pkt)
        for key, value in self.settings.list():
            pipe.write(bytearray(jdpack("z b", key, value)))
        pipe.close()

    def _handle_list_keys(self, pkt: JDPacket):
        pipe = OutPipe(self.bus, pkt)
        keys = self.settings.list_keys()
        for key in keys:
            pipe.write(bytearray(jdpack("s", key)))
        pipe.close()



# Autogenerated constants for Microphone service
from jacdac.constants import *
JD_SERVICE_CLASS_MICROPHONE = const(0x113dac86)
JD_MICROPHONE_CMD_SAMPLE = const(0x81)
JD_MICROPHONE_REG_SAMPLING_PERIOD = const(0x80)
JD_MICROPHONE_PACK_FORMATS = {
    JD_MICROPHONE_CMD_SAMPLE: "b[12] u32",
    JD_MICROPHONE_REG_SAMPLING_PERIOD: "u32"
}



# Autogenerated constants for Distance service
from enum import IntEnum
from jacdac.constants import *
from jacdac.system.constants import *
JD_SERVICE_CLASS_DISTANCE = const(0x141a6b8a)


class DistanceVariant(IntEnum):
    ULTRASONIC = const(0x1)
    INFRARED = const(0x2)
    LI_DAR = const(0x3)
    LASER = const(0x4)


JD_DISTANCE_REG_DISTANCE = const(JD_REG_READING)
JD_DISTANCE_REG_DISTANCE_ERROR = const(JD_REG_READING_ERROR)
JD_DISTANCE_REG_MIN_RANGE = const(JD_REG_MIN_READING)
JD_DISTANCE_REG_MAX_RANGE = const(JD_REG_MAX_READING)
JD_DISTANCE_REG_VARIANT = const(JD_REG_VARIANT)
JD_DISTANCE_PACK_FORMATS = {
    JD_DISTANCE_REG_DISTANCE: "u16.16",
    JD_DISTANCE_REG_DISTANCE_ERROR: "u16.16",
    JD_DISTANCE_REG_MIN_RANGE: "u16.16",
    JD_DISTANCE_REG_MAX_RANGE: "u16.16",
    JD_DISTANCE_REG_VARIANT: "u8"
}



# Autogenerated constants for LED Strip service
from enum import IntEnum
from jacdac.constants import *
from jacdac.system.constants import *
JD_SERVICE_CLASS_LED_STRIP = const(0x126f00e0)


class LedStripLightType(IntEnum):
    WS2812B_GRB = const(0x0)
    APA102 = const(0x10)
    SK9822 = const(0x11)


class LedStripVariant(IntEnum):
    STRIP = const(0x1)
    RING = const(0x2)
    STICK = const(0x3)
    JEWEL = const(0x4)
    MATRIX = const(0x5)


JD_LED_STRIP_REG_BRIGHTNESS = const(JD_REG_INTENSITY)
JD_LED_STRIP_REG_ACTUAL_BRIGHTNESS = const(0x180)
JD_LED_STRIP_REG_LIGHT_TYPE = const(0x80)
JD_LED_STRIP_REG_NUM_PIXELS = const(0x81)
JD_LED_STRIP_REG_NUM_COLUMNS = const(0x83)
JD_LED_STRIP_REG_MAX_POWER = const(JD_REG_MAX_POWER)
JD_LED_STRIP_REG_MAX_PIXELS = const(0x181)
JD_LED_STRIP_REG_NUM_REPEATS = const(0x82)
JD_LED_STRIP_REG_VARIANT = const(JD_REG_VARIANT)
JD_LED_STRIP_CMD_RUN = const(0x81)
JD_LED_STRIP_PACK_FORMATS = {
    JD_LED_STRIP_REG_BRIGHTNESS: "u0.8",
    JD_LED_STRIP_REG_ACTUAL_BRIGHTNESS: "u0.8",
    JD_LED_STRIP_REG_LIGHT_TYPE: "u8",
    JD_LED_STRIP_REG_NUM_PIXELS: "u16",
    JD_LED_STRIP_REG_NUM_COLUMNS: "u16",
    JD_LED_STRIP_REG_MAX_POWER: "u16",
    JD_LED_STRIP_REG_MAX_PIXELS: "u16",
    JD_LED_STRIP_REG_NUM_REPEATS: "u16",
    JD_LED_STRIP_REG_VARIANT: "u8",
    JD_LED_STRIP_CMD_RUN: "b"
}



# Autogenerated constants for Equivalent CO₂ service
from enum import IntEnum
from jacdac.constants import *
from jacdac.system.constants import *
JD_SERVICE_CLASS_E_CO2 = const(0x169c9dc6)


class ECO2Variant(IntEnum):
    VOC = const(0x1)
    NDIR = const(0x2)


JD_E_CO2_REG_E_CO2 = const(JD_REG_READING)
JD_E_CO2_REG_E_CO2_ERROR = const(JD_REG_READING_ERROR)
JD_E_CO2_REG_MIN_E_CO2 = const(JD_REG_MIN_READING)
JD_E_CO2_REG_MAX_E_CO2 = const(JD_REG_MAX_READING)
JD_E_CO2_REG_VARIANT = const(JD_REG_VARIANT)
JD_E_CO2_PACK_FORMATS = {
    JD_E_CO2_REG_E_CO2: "u22.10",
    JD_E_CO2_REG_E_CO2_ERROR: "u22.10",
    JD_E_CO2_REG_MIN_E_CO2: "u22.10",
    JD_E_CO2_REG_MAX_E_CO2: "u22.10",
    JD_E_CO2_REG_VARIANT: "u8"
}



# Autogenerated constants for Potentiometer service
from enum import IntEnum
from jacdac.constants import *
from jacdac.system.constants import *
JD_SERVICE_CLASS_POTENTIOMETER = const(0x1f274746)


class PotentiometerVariant(IntEnum):
    SLIDER = const(0x1)
    ROTARY = const(0x2)


JD_POTENTIOMETER_REG_POSITION = const(JD_REG_READING)
JD_POTENTIOMETER_REG_VARIANT = const(JD_REG_VARIANT)
JD_POTENTIOMETER_PACK_FORMATS = {
    JD_POTENTIOMETER_REG_POSITION: "u0.16",
    JD_POTENTIOMETER_REG_VARIANT: "u8"
}



# Autogenerated constants for Dimmer service
from enum import IntEnum
from jacdac.constants import *
from jacdac.system.constants import *
JD_SERVICE_CLASS_DIMMER = const(0x1fb02645)


class DimmerVariant(IntEnum):
    LIGHT = const(0x1)
    FAN = const(0x2)
    PUMP = const(0x3)


JD_DIMMER_REG_INTENSITY = const(JD_REG_INTENSITY)
JD_DIMMER_REG_VARIANT = const(JD_REG_VARIANT)
JD_DIMMER_PACK_FORMATS = {
    JD_DIMMER_REG_INTENSITY: "u0.16",
    JD_DIMMER_REG_VARIANT: "u8"
}



# Autogenerated file. Do not edit.
from jacdac.bus import Bus, SensorClient
from .constants import *
from typing import Optional


class ServoClient(SensorClient):
    """
    Servo is a small motor with arm that can be pointing at a specific direction.
     * Typically a servo angle is between 0° and 180° where 90° is the middle resting position.
     * 
     * The `min_pulse/max_pulse` may be read-only if the servo is permanently affixed to its Jacdac controller.
    Implements a client for the `Servo <https://microsoft.github.io/jacdac-docs/services/servo>`_ service.

    """

    def __init__(self, bus: Bus, role: str, *, missing_actual_angle_value: float = None) -> None:
        super().__init__(bus, JD_SERVICE_CLASS_SERVO, JD_SERVO_PACK_FORMATS, role)
        self.missing_actual_angle_value = missing_actual_angle_value

    @property
    def angle(self) -> Optional[float]:
        """
        Specifies the angle of the arm (request)., _: °
        """
        return self.register(JD_SERVO_REG_ANGLE).value()

    @angle.setter
    def angle(self, value: float) -> None:
        self.enabled = True
        self.register(JD_SERVO_REG_ANGLE).set_values(value)


    @property
    def enabled(self) -> Optional[bool]:
        """
        Turn the power to the servo on/off., 
        """
        return self.register(JD_SERVO_REG_ENABLED).bool_value()

    @enabled.setter
    def enabled(self, value: bool) -> None:
        self.register(JD_SERVO_REG_ENABLED).set_values(value)


    @property
    def offset(self) -> Optional[float]:
        """
        Correction applied to the angle to account for the servo arm drift., _: °
        """
        return self.register(JD_SERVO_REG_OFFSET).value()

    @offset.setter
    def offset(self, value: float) -> None:
        self.register(JD_SERVO_REG_OFFSET).set_values(value)


    @property
    def min_angle(self) -> Optional[float]:
        """
        Lowest angle that can be set, typiclly 0 °., _: °
        """
        return self.register(JD_SERVO_REG_MIN_ANGLE).value()

    @property
    def min_pulse(self) -> Optional[int]:
        """
        (Optional) The length of pulse corresponding to lowest angle., _: us
        """
        return self.register(JD_SERVO_REG_MIN_PULSE).value()

    @min_pulse.setter
    def min_pulse(self, value: int) -> None:
        self.register(JD_SERVO_REG_MIN_PULSE).set_values(value)


    @property
    def max_angle(self) -> Optional[float]:
        """
        Highest angle that can be set, typically 180°., _: °
        """
        return self.register(JD_SERVO_REG_MAX_ANGLE).value()

    @property
    def max_pulse(self) -> Optional[int]:
        """
        (Optional) The length of pulse corresponding to highest angle., _: us
        """
        return self.register(JD_SERVO_REG_MAX_PULSE).value()

    @max_pulse.setter
    def max_pulse(self, value: int) -> None:
        self.register(JD_SERVO_REG_MAX_PULSE).set_values(value)


    @property
    def stall_torque(self) -> Optional[float]:
        """
        (Optional) The servo motor will stop rotating when it is trying to move a ``stall_torque`` weight at a radial distance of ``1.0`` cm., _: kg/cm
        """
        return self.register(JD_SERVO_REG_STALL_TORQUE).value()

    @property
    def response_speed(self) -> Optional[float]:
        """
        (Optional) Time to move 60°., _: s/60°
        """
        return self.register(JD_SERVO_REG_RESPONSE_SPEED).value()

    @property
    def actual_angle(self) -> Optional[float]:
        """
        (Optional) The current physical position of the arm, if the device has a way to sense the position., _: °
        """
        self.refresh_reading()
        return self.register(JD_SERVO_REG_ACTUAL_ANGLE).value(self.missing_actual_angle_value)

    



# Autogenerated file. Do not edit.
from jacdac.bus import Bus, Client
from .constants import *
from typing import Optional


class RelayClient(Client):
    """
    A switching relay.
     * 
     * The contacts should be labelled `NO` (normally open), `COM` (common), and `NC` (normally closed).
     * When relay is energized it connects `NO` and `COM`.
     * When relay is not energized it connects `NC` and `COM`.
     * Some relays may be missing `NO` or `NC` contacts.
     * When relay module is not powered, or is in bootloader mode, it is not energized (connects `NC` and `COM`).
    Implements a client for the `Relay <https://microsoft.github.io/jacdac-docs/services/relay>`_ service.

    """

    def __init__(self, bus: Bus, role: str) -> None:
        super().__init__(bus, JD_SERVICE_CLASS_RELAY, JD_RELAY_PACK_FORMATS, role)


    @property
    def active(self) -> Optional[bool]:
        """
        Indicates whether the relay circuit is currently energized or not., 
        """
        return self.register(JD_RELAY_REG_ACTIVE).bool_value()

    @active.setter
    def active(self, value: bool) -> None:
        self.register(JD_RELAY_REG_ACTIVE).set_values(value)


    @property
    def variant(self) -> Optional[RelayVariant]:
        """
        (Optional) Describes the type of relay used., 
        """
        return self.register(JD_RELAY_REG_VARIANT).value()

    @property
    def max_switching_current(self) -> Optional[int]:
        """
        (Optional) Maximum switching current for a resistive load., _: mA
        """
        return self.register(JD_RELAY_REG_MAX_SWITCHING_CURRENT).value()

    



# Autogenerated file. Do not edit.
from jacdac.bus import Bus, SensorClient
from .constants import *
from typing import Optional


class AcidityClient(SensorClient):
    """
    A sensor measuring water acidity, commonly called pH.
    Implements a client for the `Acidity <https://microsoft.github.io/jacdac-docs/services/acidity>`_ service.

    """

    def __init__(self, bus: Bus, role: str, *, missing_acidity_value: float = None) -> None:
        super().__init__(bus, JD_SERVICE_CLASS_ACIDITY, JD_ACIDITY_PACK_FORMATS, role, preferred_interval = 5000)
        self.missing_acidity_value = missing_acidity_value

    @property
    def acidity(self) -> Optional[float]:
        """
        The acidity, pH, of water., _: pH
        """
        self.refresh_reading()
        return self.register(JD_ACIDITY_REG_ACIDITY).value(self.missing_acidity_value)

    @property
    def acidity_error(self) -> Optional[float]:
        """
        (Optional) Error on the acidity reading., _: pH
        """
        return self.register(JD_ACIDITY_REG_ACIDITY_ERROR).value()

    @property
    def min_acidity(self) -> Optional[float]:
        """
        (Optional) Lowest acidity that can be reported., _: pH
        """
        return self.register(JD_ACIDITY_REG_MIN_ACIDITY).value()

    @property
    def max_humidity(self) -> Optional[float]:
        """
        (Optional) Highest acidity that can be reported., _: pH
        """
        return self.register(JD_ACIDITY_REG_MAX_HUMIDITY).value()

    



# pyright: reportGeneralTypeIssues=false
import threading
from typing import Optional
import websocket # type: ignore
from jacdac.bus import Transport


class WebSocketTransport(Transport):
    def __init__(self, url: str):
        self.url = url
        self.ws: Optional[websocket.WebSocketApp] = None
        self.opened = False
        self.open()

    def open(self) -> None:
        self.ws = websocket.WebSocketApp(self.url,
                               on_open=self.on_open,
                               on_message=self.on_message,
                               on_error=self.on_error,
                               on_close=self.on_close)
        t = threading.Thread(target=self.ws.run_forever)  # type: ignore
        t.daemon = True
        t.start()

    def send(self, pkt: bytes) -> None:
        if self.opened:
            self.ws.send(pkt, opcode=2)  # type: ignore

    def on_message(self, _, message: bytes):
        if self.on_receive:
            self.on_receive(message)

    def on_error(self, _, error: str):
        if self.opened:
            print(error)

    def on_close(self, _, close_status_code: int, close_msg: str):
        self.opened = False

    def on_open(self, _):
        print("devtools server connected at " + self.url)
        self.opened = True



# Autogenerated file. Do not edit.
from jacdac.bus import Bus, Client, EventHandlerFn, UnsubscribeFn
from .constants import *



class SettingsClient(Client):
    """
    Non-volatile key-value storage interface for storing settings.
    Implements a client for the `Settings <https://microsoft.github.io/jacdac-docs/services/settings>`_ service.

    """

    def __init__(self, bus: Bus, role: str) -> None:
        super().__init__(bus, JD_SERVICE_CLASS_SETTINGS, JD_SETTINGS_PACK_FORMATS, role)


    def on_change(self, handler: EventHandlerFn) -> UnsubscribeFn:
        """
        Notifies that some setting have been modified.
        """
        return self.on_event(JD_SETTINGS_EV_CHANGE, handler)


    def get(self, key: str) -> None:
        """
        Get the value of given setting. If no such entry exists, the value returned is empty.
        """
        self.send_cmd_packed(JD_SETTINGS_CMD_GET, key)
    



# Autogenerated file. Do not edit.
from jacdac.bus import Bus, Client
from .constants import *
from typing import Optional


class PressureButtonClient(Client):
    """
    A pressure sensitive push-button.
    Implements a client for the `Pressure Button <https://microsoft.github.io/jacdac-docs/services/pressurebutton>`_ service.

    """

    def __init__(self, bus: Bus, role: str) -> None:
        super().__init__(bus, JD_SERVICE_CLASS_PRESSURE_BUTTON, JD_PRESSURE_BUTTON_PACK_FORMATS, role)


    @property
    def threshold(self) -> Optional[float]:
        """
        Indicates the threshold for ``up`` events., _: /
        """
        return self.register(JD_PRESSURE_BUTTON_REG_THRESHOLD).float_value(100)

    @threshold.setter
    def threshold(self, value: float) -> None:
        self.register(JD_PRESSURE_BUTTON_REG_THRESHOLD).set_values(value / 100)


    



# Autogenerated constants for Capacitive Button service
from jacdac.constants import *
from jacdac.system.constants import *
JD_SERVICE_CLASS_CAPACITIVE_BUTTON = const(0x2865adc9)
JD_CAPACITIVE_BUTTON_REG_THRESHOLD = const(JD_REG_ACTIVE_THRESHOLD)
JD_CAPACITIVE_BUTTON_CMD_CALIBRATE = const(JD_CMD_CALIBRATE)
JD_CAPACITIVE_BUTTON_PACK_FORMATS = {
    JD_CAPACITIVE_BUTTON_REG_THRESHOLD: "u0.16"
}



# Autogenerated constants for Power supply service
from jacdac.constants import *
from jacdac.system.constants import *
JD_SERVICE_CLASS_POWER_SUPPLY = const(0x1f40375f)
JD_POWER_SUPPLY_REG_ENABLED = const(JD_REG_INTENSITY)
JD_POWER_SUPPLY_REG_OUTPUT_VOLTAGE = const(JD_REG_VALUE)
JD_POWER_SUPPLY_REG_MINIMUM_VOLTAGE = const(JD_REG_MIN_VALUE)
JD_POWER_SUPPLY_REG_MAXIMUM_VOLTAGE = const(JD_REG_MAX_VALUE)
JD_POWER_SUPPLY_PACK_FORMATS = {
    JD_POWER_SUPPLY_REG_ENABLED: "u8",
    JD_POWER_SUPPLY_REG_OUTPUT_VOLTAGE: "f64",
    JD_POWER_SUPPLY_REG_MINIMUM_VOLTAGE: "f64",
    JD_POWER_SUPPLY_REG_MAXIMUM_VOLTAGE: "f64"
}



# Autogenerated file. Do not edit.
from jacdac.bus import Bus, Client
from .constants import *
from typing import Optional


class ArcadeSoundClient(Client):
    """
    A sound playing device.
     * 
     * This is typically run over an SPI connection, not regular single-wire Jacdac.
    Implements a client for the `Arcade Sound <https://microsoft.github.io/jacdac-docs/services/arcadesound>`_ service.

    """

    def __init__(self, bus: Bus, role: str) -> None:
        super().__init__(bus, JD_SERVICE_CLASS_ARCADE_SOUND, JD_ARCADE_SOUND_PACK_FORMATS, role)


    @property
    def sample_rate(self) -> Optional[float]:
        """
        Get or set playback sample rate (in samples per second).
        If you set it, read it back, as the value may be rounded up or down., _: Hz
        """
        return self.register(JD_ARCADE_SOUND_REG_SAMPLE_RATE).value()

    @sample_rate.setter
    def sample_rate(self, value: float) -> None:
        self.register(JD_ARCADE_SOUND_REG_SAMPLE_RATE).set_values(value)


    @property
    def buffer_size(self) -> Optional[int]:
        """
        The size of the internal audio buffer., _: B
        """
        return self.register(JD_ARCADE_SOUND_REG_BUFFER_SIZE).value()

    @property
    def buffer_pending(self) -> Optional[int]:
        """
        How much data is still left in the buffer to play.
        Clients should not send more data than `buffer_size - buffer_pending`,
        but can keep the `buffer_pending` as low as they want to ensure low latency
        of audio playback., _: B
        """
        return self.register(JD_ARCADE_SOUND_REG_BUFFER_PENDING).value()


    def play(self, samples: bytes) -> None:
        """
        Play samples, which are single channel, signed 16-bit little endian values.
        """
        self.send_cmd_packed(JD_ARCADE_SOUND_CMD_PLAY, samples)
    



from typing import Any, Type, List
from ..bus import Bus
from .client import ButtonClient
from time import sleep

if __name__ == '__main__':

    bus = Bus()

    def up(data: Any):
        print("up")

    def down(data: Any):
        print("down")

    def hold(data: List[Type[int]]):
        print("hold", data[0])

    btn1 = ButtonClient(bus, "btn1")
    btn1.on_up(up)
    btn1.on_down(down)
    btn1.on_hold(hold)

    while True:
        print("pressed: ", btn1.pressed)
        print("pressure: ", btn1.pressure)
        sleep(1)



# Autogenerated constants for Braille display service
from jacdac.constants import *
from jacdac.system.constants import *
JD_SERVICE_CLASS_BRAILLE_DISPLAY = const(0x13bfb7cc)
JD_BRAILLE_DISPLAY_REG_ENABLED = const(JD_REG_INTENSITY)
JD_BRAILLE_DISPLAY_REG_PATTERNS = const(JD_REG_VALUE)
JD_BRAILLE_DISPLAY_REG_LENGTH = const(0x181)
JD_BRAILLE_DISPLAY_PACK_FORMATS = {
    JD_BRAILLE_DISPLAY_REG_ENABLED: "u8",
    JD_BRAILLE_DISPLAY_REG_PATTERNS: "s",
    JD_BRAILLE_DISPLAY_REG_LENGTH: "u8"
}



# Autogenerated constants for DC Voltage Measurement service
from enum import IntEnum
from jacdac.constants import *
from jacdac.system.constants import *
JD_SERVICE_CLASS_D_CVOLTAGE_MEASUREMENT = const(0x1633ac19)


class DCVoltageMeasurementVoltageMeasurementType(IntEnum):
    ABSOLUTE = const(0x0)
    DIFFERENTIAL = const(0x1)


JD_D_CVOLTAGE_MEASUREMENT_REG_MEASUREMENT_TYPE = const(0x181)
JD_D_CVOLTAGE_MEASUREMENT_REG_MEASUREMENT_NAME = const(0x182)
JD_D_CVOLTAGE_MEASUREMENT_REG_MEASUREMENT = const(JD_REG_READING)
JD_D_CVOLTAGE_MEASUREMENT_PACK_FORMATS = {
    JD_D_CVOLTAGE_MEASUREMENT_REG_MEASUREMENT_TYPE: "u8",
    JD_D_CVOLTAGE_MEASUREMENT_REG_MEASUREMENT_NAME: "s",
    JD_D_CVOLTAGE_MEASUREMENT_REG_MEASUREMENT: "f64"
}



# Autogenerated file. Do not edit.
from jacdac.bus import Bus, Client
from .constants import *
from typing import Optional


class HidJoystickClient(Client):
    """
    Controls a HID joystick.
    Implements a client for the `HID Joystick <https://microsoft.github.io/jacdac-docs/services/hidjoystick>`_ service.

    """

    def __init__(self, bus: Bus, role: str) -> None:
        super().__init__(bus, JD_SERVICE_CLASS_HID_JOYSTICK, JD_HID_JOYSTICK_PACK_FORMATS, role)


    @property
    def button_count(self) -> Optional[int]:
        """
        Number of button report supported, 
        """
        return self.register(JD_HID_JOYSTICK_REG_BUTTON_COUNT).value()

    @property
    def buttons_analog(self) -> Optional[int]:
        """
        A bitset that indicates which button is analog., 
        """
        return self.register(JD_HID_JOYSTICK_REG_BUTTONS_ANALOG).value()

    @property
    def axis_count(self) -> Optional[int]:
        """
        Number of analog input supported, 
        """
        return self.register(JD_HID_JOYSTICK_REG_AXIS_COUNT).value()

    



# Autogenerated file. Do not edit.
from jacdac.bus import Bus, Client
from .constants import *
from typing import Optional


class SpeechSynthesisClient(Client):
    """
    A speech synthesizer
    Implements a client for the `Speech synthesis <https://microsoft.github.io/jacdac-docs/services/speechsynthesis>`_ service.

    """

    def __init__(self, bus: Bus, role: str) -> None:
        super().__init__(bus, JD_SERVICE_CLASS_SPEECH_SYNTHESIS, JD_SPEECH_SYNTHESIS_PACK_FORMATS, role)


    @property
    def enabled(self) -> Optional[bool]:
        """
        Determines if the speech engine is in a non-paused state., 
        """
        return self.register(JD_SPEECH_SYNTHESIS_REG_ENABLED).bool_value()

    @enabled.setter
    def enabled(self, value: bool) -> None:
        self.register(JD_SPEECH_SYNTHESIS_REG_ENABLED).set_values(value)


    @property
    def lang(self) -> Optional[str]:
        """
        (Optional) Language used for utterances as defined in https://www.ietf.org/rfc/bcp/bcp47.txt., 
        """
        return self.register(JD_SPEECH_SYNTHESIS_REG_LANG).value()

    @lang.setter
    def lang(self, value: str) -> None:
        self.register(JD_SPEECH_SYNTHESIS_REG_LANG).set_values(value)


    @property
    def volume(self) -> Optional[float]:
        """
        (Optional) Volume for utterances., _: /
        """
        return self.register(JD_SPEECH_SYNTHESIS_REG_VOLUME).float_value(100)

    @volume.setter
    def volume(self, value: float) -> None:
        self.register(JD_SPEECH_SYNTHESIS_REG_VOLUME).set_values(value / 100)


    @property
    def pitch(self) -> Optional[float]:
        """
        (Optional) Pitch for utterances, 
        """
        return self.register(JD_SPEECH_SYNTHESIS_REG_PITCH).value()

    @pitch.setter
    def pitch(self, value: float) -> None:
        self.register(JD_SPEECH_SYNTHESIS_REG_PITCH).set_values(value)


    @property
    def rate(self) -> Optional[float]:
        """
        (Optional) Rate for utterances, 
        """
        return self.register(JD_SPEECH_SYNTHESIS_REG_RATE).value()

    @rate.setter
    def rate(self, value: float) -> None:
        self.register(JD_SPEECH_SYNTHESIS_REG_RATE).set_values(value)



    def speak(self, text: str) -> None:
        """
        Adds an utterance to the utterance queue; it will be spoken when any other utterances queued before it have been spoken.
        """
        self.send_cmd_packed(JD_SPEECH_SYNTHESIS_CMD_SPEAK, text)

    def cancel(self, ) -> None:
        """
        Cancels current utterance and all utterances from the utterance queue.
        """
        self.send_cmd_packed(JD_SPEECH_SYNTHESIS_CMD_CANCEL, )
    



from typing import Any, Optional

from .pack import PackType, jdpack, jdunpack
from .constants import *
from .system.constants import JD_CMD_COMMAND_NOT_IMPLEMENTED
import jacdac.util as util


class JDPacket:
    """A Jacdac packet
    """

    def __init__(self, *, cmd: int = None, size: int = 0, frombytes: bytes = None, data: bytes = None, sender: Any = None) -> None:
        self.timestamp = util.now()
        if frombytes is None:
            self._header = bytearray(JD_SERIAL_HEADER_SIZE)
            self.data = bytearray(data or size)
        else:
            self._header = bytearray(frombytes[0:JD_SERIAL_HEADER_SIZE])
            self.data = bytearray(frombytes[JD_SERIAL_HEADER_SIZE:])
        if cmd is not None:
            self.service_command = cmd
        self.sender = sender

    @staticmethod
    def packed(cmd: int, fmt: str, *args: PackType):
        return JDPacket(cmd=cmd, data=jdpack(fmt, *args))

    def unpack(self, fmt: str):
        return jdunpack(self.data, fmt)

    @property
    def service_command(self):
        return util.u16(self._header, 14)

    @service_command.setter
    def service_command(self, cmd: int):
        util.set_u16(self._header, 14, cmd)

    @property
    def device_id(self) -> str:
        return util.buf2hex(self._header[4:12])

    @device_id.setter
    def device_id(self, id_str: str):
        id = util.hex2buf(id_str)
        if len(id) != 8:
            raise ValueError()
        self._header[4:12] = id

    @property
    def packet_flags(self):
        return self._header[3]

    @property
    def multicommand_class(self):
        if self.packet_flags & JD_FRAME_FLAG_IDENTIFIER_IS_SERVICE_CLASS:
            return util.u32(self._header, 4)
        else:
            return None

    @property
    def size(self):
        return self._header[12]

    @property
    def requires_ack(self):
        return (self.packet_flags & JD_FRAME_FLAG_ACK_REQUESTED) != 0

    @requires_ack.setter
    def requires_ack(self, val: bool):
        if val != self.requires_ack:
            self._header[3] ^= JD_FRAME_FLAG_ACK_REQUESTED

    @property
    def service_index(self):
        return self._header[13] & JD_SERVICE_INDEX_MASK

    @property
    def is_regular_service(self):
        return self.service_index <= 58

    @service_index.setter
    def service_index(self, val:  Optional[int]):
        if val is None:
            raise ValueError("service_index not set")
        self._header[13] = (self._header[13] & JD_SERVICE_INDEX_INV_MASK) | val

    @property
    def crc(self):
        return util.u16(self._header, 0)

    @property
    def is_event(self):
        return self.is_report and self.is_regular_service and (self.service_command & CMD_EVENT_MASK) != 0

    @property
    def event_code(self):
        assert self.is_event
        return self.service_command & CMD_EVENT_CODE_MASK

    @property
    def event_counter(self):
        assert self.is_event
        return (self.service_command >> CMD_EVENT_COUNTER_POS) & CMD_EVENT_COUNTER_MASK

    @property
    def is_reg_set(self):
        return self.is_regular_service and self.service_command >> 12 == CMD_SET_REG >> 12

    @property
    def is_reg_get(self):
        return self.is_regular_service and self.service_command >> 12 == CMD_GET_REG >> 12

    @property
    def reg_code(self):
        return self.service_command & CMD_REG_MASK

    @property
    def data(self):
        return self._data

    @data.setter
    def data(self, buf: bytearray):
        if len(buf) > JD_SERIAL_MAX_PAYLOAD_SIZE:
            raise ValueError("Too big")
        self._header[12] = len(buf)
        self._data = buf

    @property
    def is_command(self):
        return (self.packet_flags & JD_FRAME_FLAG_COMMAND) != 0

    @property
    def is_report(self):
        return (self.packet_flags & JD_FRAME_FLAG_COMMAND) == 0

    def to_string(self):
        msg = "{}/{}[{}]: {} sz={}".format(
            util.short_id(self._header[4:12]),
            self.service_index,
            self.packet_flags,
            util.hex_num(self.service_command, 4),
            self.size)
        if self.size < 20:
            msg += ": " + util.buf2hex(self.data)
        else:
            msg += ": " + util.buf2hex(self.data[0:20]) + "..."
        return msg

    def __str__(self):
        return "<JDPacket {}>".format(self.to_string())

    def not_implemented(self):
        return JDPacket.packed(JD_CMD_COMMAND_NOT_IMPLEMENTED, "u16 u16", self.service_command, self.crc)



# Autogenerated file. Do not edit.
from jacdac.bus import Bus, SensorClient
from .constants import *
from typing import Optional


class RainGaugeClient(SensorClient):
    """
    Measures the amount of liquid precipitation over an area in a predefined period of time.
    Implements a client for the `Rain gauge <https://microsoft.github.io/jacdac-docs/services/raingauge>`_ service.

    """

    def __init__(self, bus: Bus, role: str, *, missing_precipitation_value: float = None) -> None:
        super().__init__(bus, JD_SERVICE_CLASS_RAIN_GAUGE, JD_RAIN_GAUGE_PACK_FORMATS, role, preferred_interval = 60000)
        self.missing_precipitation_value = missing_precipitation_value

    @property
    def precipitation(self) -> Optional[float]:
        """
        Total precipitation recorded so far., _: mm
        """
        self.refresh_reading()
        return self.register(JD_RAIN_GAUGE_REG_PRECIPITATION).value(self.missing_precipitation_value)

    @property
    def precipitation_precision(self) -> Optional[float]:
        """
        (Optional) Typically the amount of rain needed for tipping the bucket., _: mm
        """
        return self.register(JD_RAIN_GAUGE_REG_PRECIPITATION_PRECISION).value()

    



# Autogenerated constants for Arcade Sound service
from jacdac.constants import *
JD_SERVICE_CLASS_ARCADE_SOUND = const(0x1fc63606)
JD_ARCADE_SOUND_CMD_PLAY = const(0x80)
JD_ARCADE_SOUND_REG_SAMPLE_RATE = const(0x80)
JD_ARCADE_SOUND_REG_BUFFER_SIZE = const(0x180)
JD_ARCADE_SOUND_REG_BUFFER_PENDING = const(0x181)
JD_ARCADE_SOUND_PACK_FORMATS = {
    JD_ARCADE_SOUND_CMD_PLAY: "b",
    JD_ARCADE_SOUND_REG_SAMPLE_RATE: "u22.10",
    JD_ARCADE_SOUND_REG_BUFFER_SIZE: "u32",
    JD_ARCADE_SOUND_REG_BUFFER_PENDING: "u32"
}



# Autogenerated constants for Vibration motor service
from jacdac.constants import *
JD_SERVICE_CLASS_VIBRATION_MOTOR = const(0x183fc4a2)
JD_VIBRATION_MOTOR_CMD_VIBRATE = const(0x80)
JD_VIBRATION_MOTOR_PACK_FORMATS = {
    JD_VIBRATION_MOTOR_CMD_VIBRATE: "r: u8 u0.8"
}



# Autogenerated file. Do not edit.
from jacdac.bus import Bus, SensorClient, EventHandlerFn, UnsubscribeFn
from .constants import *
from typing import Optional


class ButtonClientBase(SensorClient):
    """
    A push-button, which returns to inactive position when not operated anymore.
    Implements a client for the `Button <https://microsoft.github.io/jacdac-docs/services/button>`_ service.

    """

    def __init__(self, bus: Bus, role: str, *, missing_pressure_value: float = None) -> None:
        super().__init__(bus, JD_SERVICE_CLASS_BUTTON, JD_BUTTON_PACK_FORMATS, role)
        self.missing_pressure_value = missing_pressure_value

    @property
    def pressure(self) -> Optional[float]:
        """
        Indicates the pressure state of the button, where `0` is open., _: /
        """
        self.refresh_reading()
        return self.register(JD_BUTTON_REG_PRESSURE).float_value(self.missing_pressure_value, 100)

    @property
    def analog(self) -> Optional[bool]:
        """
        (Optional) Indicates if the button provides analog `pressure` readings., 
        """
        return self.register(JD_BUTTON_REG_ANALOG).bool_value()

    def on_down(self, handler: EventHandlerFn) -> UnsubscribeFn:
        """
        Emitted when button goes from inactive to active.
        """
        return self.on_event(JD_BUTTON_EV_DOWN, handler)

    def on_up(self, handler: EventHandlerFn) -> UnsubscribeFn:
        """
        Emitted when button goes from active to inactive. The 'time' parameter
        records the amount of time between the down and up events.
        """
        return self.on_event(JD_BUTTON_EV_UP, handler)

    def on_hold(self, handler: EventHandlerFn) -> UnsubscribeFn:
        """
        Emitted when the press time is greater than 500ms, and then at least every 500ms
        as long as the button remains pressed. The 'time' parameter records the the amount of time
        that the button has been held (since the down event).
        """
        return self.on_event(JD_BUTTON_EV_HOLD, handler)

    



# Autogenerated file. Do not edit.
from jacdac.bus import Bus, Client
from .constants import *
from typing import Optional, Tuple


class LedSingleClient(Client):
    """
    A controller for 1 or more monochrome or RGB LEDs connected in parallel.
    Implements a client for the `LED Single <https://microsoft.github.io/jacdac-docs/services/ledsingle>`_ service.

    """

    def __init__(self, bus: Bus, role: str) -> None:
        super().__init__(bus, JD_SERVICE_CLASS_LED_SINGLE, JD_LED_SINGLE_PACK_FORMATS, role)


    @property
    def color(self) -> Optional[Tuple[int, int, int]]:
        """
        The current color of the LED., 
        """
        return self.register(JD_LED_SINGLE_REG_COLOR).value()

    @property
    def max_power(self) -> Optional[int]:
        """
        (Optional) Limit the power drawn by the light-strip (and controller)., _: mA
        """
        return self.register(JD_LED_SINGLE_REG_MAX_POWER).value()

    @max_power.setter
    def max_power(self, value: int) -> None:
        self.register(JD_LED_SINGLE_REG_MAX_POWER).set_values(value)


    @property
    def led_count(self) -> Optional[int]:
        """
        (Optional) If known, specifies the number of LEDs in parallel on this device., 
        """
        return self.register(JD_LED_SINGLE_REG_LED_COUNT).value()

    @property
    def wave_length(self) -> Optional[int]:
        """
        (Optional) If monochrome LED, specifies the wave length of the LED., _: nm
        """
        return self.register(JD_LED_SINGLE_REG_WAVE_LENGTH).value()

    @property
    def luminous_intensity(self) -> Optional[int]:
        """
        (Optional) The luminous intensity of the LED, at full value, in micro candella., _: mcd
        """
        return self.register(JD_LED_SINGLE_REG_LUMINOUS_INTENSITY).value()

    @property
    def variant(self) -> Optional[LedSingleVariant]:
        """
        (Optional) The physical type of LED., 
        """
        return self.register(JD_LED_SINGLE_REG_VARIANT).value()


    def animate(self, to_red: int, to_green: int, to_blue: int, speed: int) -> None:
        """
        This has the same semantics as `set_status_light` in the control service.
        """
        self.send_cmd_packed(JD_LED_SINGLE_CMD_ANIMATE, to_red, to_green, to_blue, speed)
    



# Autogenerated file. Do not edit.
from jacdac.bus import Bus, Client
from .constants import *
from typing import Optional


class MidiOutputClient(Client):
    """
    A MIDI output device.
    Implements a client for the `MIDI output <https://microsoft.github.io/jacdac-docs/services/midioutput>`_ service.

    """

    def __init__(self, bus: Bus, role: str) -> None:
        super().__init__(bus, JD_SERVICE_CLASS_MIDI_OUTPUT, JD_MIDI_OUTPUT_PACK_FORMATS, role)


    @property
    def enabled(self) -> Optional[bool]:
        """
        Opens or closes the port to the MIDI device, 
        """
        return self.register(JD_MIDI_OUTPUT_REG_ENABLED).bool_value()

    @enabled.setter
    def enabled(self, value: bool) -> None:
        self.register(JD_MIDI_OUTPUT_REG_ENABLED).set_values(value)



    def clear(self, ) -> None:
        """
        Clears any pending send data that has not yet been sent from the MIDIOutput's queue.
        """
        self.send_cmd_packed(JD_MIDI_OUTPUT_CMD_CLEAR, )

    def send(self, data: bytes) -> None:
        """
        Enqueues the message to be sent to the corresponding MIDI port
        """
        self.send_cmd_packed(JD_MIDI_OUTPUT_CMD_SEND, data)
    



# Autogenerated constants for Dot Matrix service
from enum import IntEnum
from jacdac.constants import *
from jacdac.system.constants import *
JD_SERVICE_CLASS_DOT_MATRIX = const(0x110d154b)


class DotMatrixVariant(IntEnum):
    LED = const(0x1)
    BRAILLE = const(0x2)


JD_DOT_MATRIX_REG_DOTS = const(JD_REG_VALUE)
JD_DOT_MATRIX_REG_BRIGHTNESS = const(JD_REG_INTENSITY)
JD_DOT_MATRIX_REG_ROWS = const(0x181)
JD_DOT_MATRIX_REG_COLUMNS = const(0x182)
JD_DOT_MATRIX_REG_VARIANT = const(JD_REG_VARIANT)
JD_DOT_MATRIX_PACK_FORMATS = {
    JD_DOT_MATRIX_REG_DOTS: "b",
    JD_DOT_MATRIX_REG_BRIGHTNESS: "u0.8",
    JD_DOT_MATRIX_REG_ROWS: "u16",
    JD_DOT_MATRIX_REG_COLUMNS: "u16",
    JD_DOT_MATRIX_REG_VARIANT: "u8"
}



# Autogenerated file. Do not edit.
from jacdac.bus import Bus, Client
from .constants import *
from typing import Optional


class BitRadioClient(Client):
    """
    Support for sending and receiving packets using the [Bit Radio protocol](https://github.com/microsoft/pxt-common-packages/blob/master/libs/radio/docs/reference/radio.md), typically used between micro:bit devices.
    Implements a client for the `bit:radio <https://microsoft.github.io/jacdac-docs/services/bitradio>`_ service.

    """

    def __init__(self, bus: Bus, role: str) -> None:
        super().__init__(bus, JD_SERVICE_CLASS_BIT_RADIO, JD_BIT_RADIO_PACK_FORMATS, role)


    @property
    def enabled(self) -> Optional[bool]:
        """
        Turns on/off the radio antenna., 
        """
        return self.register(JD_BIT_RADIO_REG_ENABLED).bool_value()

    @enabled.setter
    def enabled(self, value: bool) -> None:
        self.register(JD_BIT_RADIO_REG_ENABLED).set_values(value)


    @property
    def group(self) -> Optional[int]:
        """
        Group used to filter packets, 
        """
        return self.register(JD_BIT_RADIO_REG_GROUP).value()

    @group.setter
    def group(self, value: int) -> None:
        self.register(JD_BIT_RADIO_REG_GROUP).set_values(value)


    @property
    def transmission_power(self) -> Optional[int]:
        """
        Antenna power to increase or decrease range., 
        """
        return self.register(JD_BIT_RADIO_REG_TRANSMISSION_POWER).value()

    @transmission_power.setter
    def transmission_power(self, value: int) -> None:
        self.register(JD_BIT_RADIO_REG_TRANSMISSION_POWER).set_values(value)


    @property
    def frequency_band(self) -> Optional[int]:
        """
        Change the transmission and reception band of the radio to the given channel., 
        """
        return self.register(JD_BIT_RADIO_REG_FREQUENCY_BAND).value()

    @frequency_band.setter
    def frequency_band(self, value: int) -> None:
        self.register(JD_BIT_RADIO_REG_FREQUENCY_BAND).set_values(value)



    def send_string(self, message: str) -> None:
        """
        Sends a string payload as a radio message, maximum 18 characters.
        """
        self.send_cmd_packed(JD_BIT_RADIO_CMD_SEND_STRING, message)

    def send_number(self, value: float) -> None:
        """
        Sends a double precision number payload as a radio message
        """
        self.send_cmd_packed(JD_BIT_RADIO_CMD_SEND_NUMBER, value)

    def send_value(self, value: float, name: str) -> None:
        """
        Sends a double precision number and a name payload as a radio message
        """
        self.send_cmd_packed(JD_BIT_RADIO_CMD_SEND_VALUE, value, name)

    def send_buffer(self, data: bytes) -> None:
        """
        Sends a payload of bytes as a radio message
        """
        self.send_cmd_packed(JD_BIT_RADIO_CMD_SEND_BUFFER, data)
    



# Autogenerated file. Do not edit.
from jacdac.bus import Bus, Client
from .constants import *
from typing import Optional


class DimmerClient(Client):
    """
    A light or fan controller that dims the current on a line.
    Implements a client for the `Dimmer <https://microsoft.github.io/jacdac-docs/services/dimmer>`_ service.

    """

    def __init__(self, bus: Bus, role: str) -> None:
        super().__init__(bus, JD_SERVICE_CLASS_DIMMER, JD_DIMMER_PACK_FORMATS, role)


    @property
    def intensity(self) -> Optional[float]:
        """
        The intensity of the current. Set to ``0`` to turn off completely the current., _: /
        """
        return self.register(JD_DIMMER_REG_INTENSITY).float_value(100)

    @intensity.setter
    def intensity(self, value: float) -> None:
        self.register(JD_DIMMER_REG_INTENSITY).set_values(value / 100)


    @property
    def variant(self) -> Optional[DimmerVariant]:
        """
        (Optional) The type of physical device, 
        """
        return self.register(JD_DIMMER_REG_VARIANT).value()

    



# Autogenerated file. Do not edit.
from jacdac.bus import Bus, Client
from .constants import *



class TcpClient(Client):
    """
    Data transfer over TCP/IP and TLS stream sockets.
    Implements a client for the `TCP <https://microsoft.github.io/jacdac-docs/services/tcp>`_ service.

    """

    def __init__(self, bus: Bus, role: str) -> None:
        super().__init__(bus, JD_SERVICE_CLASS_TCP, JD_TCP_PACK_FORMATS, role)


    



# Autogenerated constants for Water level service
from enum import IntEnum
from jacdac.constants import *
from jacdac.system.constants import *
JD_SERVICE_CLASS_WATER_LEVEL = const(0x147b62ed)


class WaterLevelVariant(IntEnum):
    RESISTIVE = const(0x1)
    CONTACT_PHOTO_ELECTRIC = const(0x2)
    NON_CONTACT_PHOTO_ELECTRIC = const(0x3)


JD_WATER_LEVEL_REG_LEVEL = const(JD_REG_READING)
JD_WATER_LEVEL_REG_LEVEL_ERROR = const(JD_REG_READING_ERROR)
JD_WATER_LEVEL_REG_VARIANT = const(JD_REG_VARIANT)
JD_WATER_LEVEL_PACK_FORMATS = {
    JD_WATER_LEVEL_REG_LEVEL: "u0.16",
    JD_WATER_LEVEL_REG_LEVEL_ERROR: "u0.16",
    JD_WATER_LEVEL_REG_VARIANT: "u8"
}



from typing import Any
from ..bus import Bus
from .client import RelayClient
from time import sleep

if __name__ == '__main__':
    bus = Bus()

    def active(data: Any):
        print("active")

    def inactive(data: Any):
        print("inactive")

    rel = RelayClient(bus, "relay")

    while True:
        rel.active = False
        sleep(1)
        print("open: ", rel.active)
        rel.active = True
        sleep(1)
        print("closed: ", rel.active)



from ..bus import SensorServer, Bus
from ..packet import JDPacket
from .constants import *
from time import localtime


class RealTimeClockServer(SensorServer):
    """A server for the real time clock service using Python time."""

    def __init__(self, bus: Bus, *, instance_name: str = None) -> None:
        super().__init__(bus, JD_SERVICE_CLASS_REAL_TIME_CLOCK,
                         instance_name=instance_name,
                         streaming_interval=1000,
                         streaming_preferred_interval=1000)

    def send_reading(self):
        t = localtime()
        fmt = JD_REAL_TIME_CLOCK_PACK_FORMATS[JD_REAL_TIME_CLOCK_REG_LOCAL_TIME]
        self.send_report(JDPacket.packed(
            JD_GET(JD_REAL_TIME_CLOCK_REG_LOCAL_TIME), fmt, t.tm_year, t.tm_mon, t.tm_mday, t.tm_wday, t.tm_hour, t.tm_min, t.tm_sec))

    def handle_packet(self, pkt: JDPacket):
        cmd = pkt.service_command
        if cmd == JD_GET(JD_REAL_TIME_CLOCK_REG_LOCAL_TIME):
            self.send_reading()
        elif cmd == JD_GET(JD_REAL_TIME_CLOCK_REG_VARIANT):
            fmt = JD_REAL_TIME_CLOCK_PACK_FORMATS[JD_REAL_TIME_CLOCK_REG_VARIANT]
            self.send_report(JDPacket.packed(
                cmd, fmt, RealTimeClockVariant.COMPUTER))
        elif cmd == JD_GET(JD_REAL_TIME_CLOCK_REG_DRIFT):
            self.send_report(pkt.not_implemented())
        else:
            super().handle_packet(pkt)



# Autogenerated constants for Matrix Keypad service
from enum import IntEnum
from jacdac.constants import *
from jacdac.system.constants import *
JD_SERVICE_CLASS_MATRIX_KEYPAD = const(0x13062dc8)


class MatrixKeypadVariant(IntEnum):
    MEMBRANE = const(0x1)
    KEYBOARD = const(0x2)
    ELASTOMER = const(0x3)
    ELASTOMER_LEDPIXEL = const(0x4)


JD_MATRIX_KEYPAD_REG_PRESSED = const(JD_REG_READING)
JD_MATRIX_KEYPAD_REG_ROWS = const(0x180)
JD_MATRIX_KEYPAD_REG_COLUMNS = const(0x181)
JD_MATRIX_KEYPAD_REG_LABELS = const(0x182)
JD_MATRIX_KEYPAD_REG_VARIANT = const(JD_REG_VARIANT)
JD_MATRIX_KEYPAD_EV_DOWN = const(JD_EV_ACTIVE)
JD_MATRIX_KEYPAD_EV_UP = const(JD_EV_INACTIVE)
JD_MATRIX_KEYPAD_EV_CLICK = const(0x80)
JD_MATRIX_KEYPAD_EV_LONG_CLICK = const(0x81)
JD_MATRIX_KEYPAD_PACK_FORMATS = {
    JD_MATRIX_KEYPAD_REG_PRESSED: "r: u8",
    JD_MATRIX_KEYPAD_REG_ROWS: "u8",
    JD_MATRIX_KEYPAD_REG_COLUMNS: "u8",
    JD_MATRIX_KEYPAD_REG_LABELS: "r: z",
    JD_MATRIX_KEYPAD_REG_VARIANT: "u8",
    JD_MATRIX_KEYPAD_EV_DOWN: "u8",
    JD_MATRIX_KEYPAD_EV_UP: "u8",
    JD_MATRIX_KEYPAD_EV_CLICK: "u8",
    JD_MATRIX_KEYPAD_EV_LONG_CLICK: "u8"
}



# Autogenerated file. Do not edit.
from jacdac.bus import Bus, Client
from .constants import *
from typing import Optional


class MicrophoneClient(Client):
    """
    A single-channel microphone.
    Implements a client for the `Microphone <https://microsoft.github.io/jacdac-docs/services/microphone>`_ service.

    """

    def __init__(self, bus: Bus, role: str) -> None:
        super().__init__(bus, JD_SERVICE_CLASS_MICROPHONE, JD_MICROPHONE_PACK_FORMATS, role)


    @property
    def sampling_period(self) -> Optional[int]:
        """
        Get or set microphone sampling period.
        Sampling rate is `1_000_000 / sampling_period Hz`., _: us
        """
        return self.register(JD_MICROPHONE_REG_SAMPLING_PERIOD).value()

    @sampling_period.setter
    def sampling_period(self, value: int) -> None:
        self.register(JD_MICROPHONE_REG_SAMPLING_PERIOD).set_values(value)


    



# Autogenerated constants for Traffic Light service
from jacdac.constants import *
JD_SERVICE_CLASS_TRAFFIC_LIGHT = const(0x15c38d9b)
JD_TRAFFIC_LIGHT_REG_RED = const(0x80)
JD_TRAFFIC_LIGHT_REG_YELLOW = const(0x81)
JD_TRAFFIC_LIGHT_REG_GREEN = const(0x82)
JD_TRAFFIC_LIGHT_PACK_FORMATS = {
    JD_TRAFFIC_LIGHT_REG_RED: "u8",
    JD_TRAFFIC_LIGHT_REG_YELLOW: "u8",
    JD_TRAFFIC_LIGHT_REG_GREEN: "u8"
}



# Autogenerated constants for Bridge service
from jacdac.constants import *
from jacdac.system.constants import *
JD_SERVICE_CLASS_BRIDGE = const(0x1fe5b46f)
JD_BRIDGE_REG_ENABLED = const(JD_REG_INTENSITY)
JD_BRIDGE_PACK_FORMATS = {
    JD_BRIDGE_REG_ENABLED: "u8"
}



# Autogenerated constants for Air Quality Index service
from jacdac.constants import *
from jacdac.system.constants import *
JD_SERVICE_CLASS_AIR_QUALITY_INDEX = const(0x14ac6ed6)
JD_AIR_QUALITY_INDEX_REG_AQI_INDEX = const(JD_REG_READING)
JD_AIR_QUALITY_INDEX_REG_AQI_INDEX_ERROR = const(JD_REG_READING_ERROR)
JD_AIR_QUALITY_INDEX_REG_MIN_AQI_INDEX = const(JD_REG_MIN_READING)
JD_AIR_QUALITY_INDEX_REG_MAX_AQI_INDEX = const(JD_REG_MAX_READING)
JD_AIR_QUALITY_INDEX_PACK_FORMATS = {
    JD_AIR_QUALITY_INDEX_REG_AQI_INDEX: "u16.16",
    JD_AIR_QUALITY_INDEX_REG_AQI_INDEX_ERROR: "u16.16",
    JD_AIR_QUALITY_INDEX_REG_MIN_AQI_INDEX: "u16.16",
    JD_AIR_QUALITY_INDEX_REG_MAX_AQI_INDEX: "u16.16"
}



# Autogenerated file. Do not edit.
from jacdac.bus import Bus, SensorClient
from .constants import *
from typing import Optional


class DcVoltageMeasurementClient(SensorClient):
    """
    A service that reports a voltage measurement.
    Implements a client for the `DC Voltage Measurement <https://microsoft.github.io/jacdac-docs/services/dcvoltagemeasurement>`_ service.

    """

    def __init__(self, bus: Bus, role: str, *, missing_measurement_value: float = None) -> None:
        super().__init__(bus, JD_SERVICE_CLASS_DC_VOLTAGE_MEASUREMENT, JD_DC_VOLTAGE_MEASUREMENT_PACK_FORMATS, role)
        self.missing_measurement_value = missing_measurement_value

    @property
    def measurement_type(self) -> Optional[DcVoltageMeasurementVoltageMeasurementType]:
        """
        The type of measurement that is taking place. Absolute results are measured with respect to ground, whereas differential results are measured against another signal that is not ground., 
        """
        return self.register(JD_DC_VOLTAGE_MEASUREMENT_REG_MEASUREMENT_TYPE).value()

    @property
    def measurement_name(self) -> Optional[str]:
        """
        A string containing the net name that is being measured e.g. `POWER_DUT` or a reference e.g. `DIFF_DEV1_DEV2`. These constants can be used to identify a measurement from client code., 
        """
        return self.register(JD_DC_VOLTAGE_MEASUREMENT_REG_MEASUREMENT_NAME).value()

    @property
    def measurement(self) -> Optional[float]:
        """
        The voltage measurement., _: V
        """
        self.refresh_reading()
        return self.register(JD_DC_VOLTAGE_MEASUREMENT_REG_MEASUREMENT).value(self.missing_measurement_value)

    



# pyright: reportGeneralTypeIssues=false
from asyncio import get_event_loop
from websockets import serve
from http.server import BaseHTTPRequestHandler, HTTPServer
from requests import get
from threading import Thread
from sys import argv

print("Jacdac DevTools (Python)")

internet = "--internet" in argv
HOST = '0.0.0.0' if internet else 'localhost'
WS_PORT = 8081
HTTP_PORT = 8082
clients = []
proxy_source: bytes

if internet:
    print("WARNING: server bound to all network interfaces")
else:
    print("use --internet to bind server to all network interfaces")

class Handler(BaseHTTPRequestHandler) :
        def do_HEAD(self):
            self.send_response(200)    
        def do_GET(self) :
            if self.path == "/":
                self.send_response(200)
                self.send_header('Content-Type', 'text/html')
                self.send_header('Cache-Control', 'no-cache')
                self.send_header("Content-Length", str(len(proxy_source)))
                self.end_headers()
                self.wfile.write(proxy_source)
            else:
                self.send_error(404)

async def proxy(websocket, path: str):
    clients.append(websocket)
    print("client connected (%d clients)" % len(clients))
    ## listen to websocket client until it closesw
    try:
        while websocket.open:
            frame: bytes = await websocket.recv()
            if len(frame) == 0:
                continue
            # dispatch to other clients
            cs = clients.copy() # avoid races
            for client in cs:
                if client != websocket:
                    try:
                        await client.send(frame)
                    except:
                        print("client receive error")
    except:
        print("client receive error")
    finally:
        # remove from clients
        clients.remove(websocket)
        print("client disconnected (%d clients)" % len(clients))
        
# get proxy source
resp = get('https://microsoft.github.io/jacdac-docs/devtools/proxy')
if not resp.ok:
    raise RuntimeError("proxy download failed")

proxy_source = resp.text.encode('utf-8')

def web():
    print("local web: http://" + HOST + ":" + str(HTTP_PORT))
    http_server = HTTPServer( (HOST, HTTP_PORT), Handler )
    http_server.serve_forever()

def ws():
    # start web socket server
    print("websockets: ws://" + HOST + ":" + str(WS_PORT))
    ws_server = serve(proxy, HOST, WS_PORT)
    get_event_loop().run_until_complete(ws_server)
    get_event_loop().run_forever()

# start http server
thread = Thread(target = web)
thread.daemon = True
thread.start()

# start http server
ws()



# Autogenerated constants for Logger service
from enum import IntEnum
from jacdac.constants import *
JD_SERVICE_CLASS_LOGGER = const(0x12dc1fca)


class LoggerPriority(IntEnum):
    DEBUG = const(0x0)
    LOG = const(0x1)
    WARNING = const(0x2)
    ERROR = const(0x3)
    SILENT = const(0x4)


JD_LOGGER_REG_MIN_PRIORITY = const(0x80)
JD_LOGGER_CMD_DEBUG = const(0x80)
JD_LOGGER_CMD_LOG = const(0x81)
JD_LOGGER_CMD_WARN = const(0x82)
JD_LOGGER_CMD_ERROR = const(0x83)
JD_LOGGER_PACK_FORMATS = {
    JD_LOGGER_REG_MIN_PRIORITY: "u8",
    JD_LOGGER_CMD_DEBUG: "s",
    JD_LOGGER_CMD_LOG: "s",
    JD_LOGGER_CMD_WARN: "s",
    JD_LOGGER_CMD_ERROR: "s"
}



# Autogenerated file. Do not edit.
from jacdac.bus import Bus, SensorClient
from .constants import *
from typing import Optional, Tuple


class RealTimeClockClient(SensorClient):
    """
    Real time clock to support collecting data with precise time stamps.
    Implements a client for the `Real time clock <https://microsoft.github.io/jacdac-docs/services/realtimeclock>`_ service.

    """

    def __init__(self, bus: Bus, role: str, *, missing_local_time_value: Tuple[int, int, int, int, int, int, int] = None) -> None:
        super().__init__(bus, JD_SERVICE_CLASS_REAL_TIME_CLOCK, JD_REAL_TIME_CLOCK_PACK_FORMATS, role, preferred_interval = 1000)
        self.missing_local_time_value = missing_local_time_value

    @property
    def local_time(self) -> Optional[Tuple[int, int, int, int, int, int, int]]:
        """
        Current time in 24h representation.
        
        -   `day_of_month` is day of the month, starting at `1`
        -   `day_of_week` is day of the week, starting at `1` as monday. Default streaming period is 1 second., 
        """
        self.refresh_reading()
        return self.register(JD_REAL_TIME_CLOCK_REG_LOCAL_TIME).value(self.missing_local_time_value)

    @property
    def drift(self) -> Optional[float]:
        """
        (Optional) Time drift since the last call to the `set_time` command., _: s
        """
        return self.register(JD_REAL_TIME_CLOCK_REG_DRIFT).value()

    @property
    def precision(self) -> Optional[float]:
        """
        (Optional) Error on the clock, in parts per million of seconds., _: ppm
        """
        return self.register(JD_REAL_TIME_CLOCK_REG_PRECISION).value()

    @property
    def variant(self) -> Optional[RealTimeClockVariant]:
        """
        (Optional) The type of physical clock used by the sensor., 
        """
        return self.register(JD_REAL_TIME_CLOCK_REG_VARIANT).value()


    def set_time(self, year: int, month: int, day_of_month: int, day_of_week: int, hour: int, min: int, sec: int) -> None:
        """
        Sets the current time and resets the error.
        """
        self.send_cmd_packed(JD_REAL_TIME_CLOCK_CMD_SET_TIME, year, month, day_of_month, day_of_week, hour, min, sec)
    



# Autogenerated file. Do not edit.
from jacdac.bus import Bus, Client
from .constants import *
from typing import Optional


class SoundPlayerClient(Client):
    """
    A device that can play various sounds stored locally. This service is typically paired with a ``storage`` service for storing sounds.
    Implements a client for the `Sound player <https://microsoft.github.io/jacdac-docs/services/soundplayer>`_ service.

    """

    def __init__(self, bus: Bus, role: str) -> None:
        super().__init__(bus, JD_SERVICE_CLASS_SOUND_PLAYER, JD_SOUND_PLAYER_PACK_FORMATS, role)


    @property
    def volume(self) -> Optional[float]:
        """
        (Optional) Global volume of the output. ``0`` means completely off. This volume is mixed with each play volumes., _: /
        """
        return self.register(JD_SOUND_PLAYER_REG_VOLUME).float_value(100)

    @volume.setter
    def volume(self, value: float) -> None:
        self.register(JD_SOUND_PLAYER_REG_VOLUME).set_values(value / 100)



    def play(self, name: str) -> None:
        """
        Starts playing a sound.
        """
        self.send_cmd_packed(JD_SOUND_PLAYER_CMD_PLAY, name)

    def cancel(self, ) -> None:
        """
        Cancel any sound playing.
        """
        self.send_cmd_packed(JD_SOUND_PLAYER_CMD_CANCEL, )
    



# Autogenerated constants for HID Joystick service
from jacdac.constants import *
JD_SERVICE_CLASS_HID_JOYSTICK = const(0x1a112155)
JD_HID_JOYSTICK_REG_BUTTON_COUNT = const(0x180)
JD_HID_JOYSTICK_REG_BUTTONS_ANALOG = const(0x181)
JD_HID_JOYSTICK_REG_AXIS_COUNT = const(0x182)
JD_HID_JOYSTICK_CMD_SET_BUTTONS = const(0x80)
JD_HID_JOYSTICK_CMD_SET_AXIS = const(0x81)
JD_HID_JOYSTICK_PACK_FORMATS = {
    JD_HID_JOYSTICK_REG_BUTTON_COUNT: "u8",
    JD_HID_JOYSTICK_REG_BUTTONS_ANALOG: "u32",
    JD_HID_JOYSTICK_REG_AXIS_COUNT: "u8",
    JD_HID_JOYSTICK_CMD_SET_BUTTONS: "r: u0.8",
    JD_HID_JOYSTICK_CMD_SET_AXIS: "r: i1.15"
}



# Autogenerated file. Do not edit.
from jacdac.bus import Bus, SensorClient
from .constants import *
from typing import Optional


class AirQualityIndexClient(SensorClient):
    """
    The Air Quality Index is a measure of how clean or polluted air is. From min, good quality, to high, low quality.
     * The range of AQI may vary between countries (https://en.wikipedia.org/wiki/Air_quality_index).
    Implements a client for the `Air Quality Index <https://microsoft.github.io/jacdac-docs/services/airqualityindex>`_ service.

    """

    def __init__(self, bus: Bus, role: str, *, missing_aqi_index_value: float = None) -> None:
        super().__init__(bus, JD_SERVICE_CLASS_AIR_QUALITY_INDEX, JD_AIR_QUALITY_INDEX_PACK_FORMATS, role, preferred_interval = 60000)
        self.missing_aqi_index_value = missing_aqi_index_value

    @property
    def aqi_index(self) -> Optional[float]:
        """
        Air quality index, typically refreshed every second., _: AQI
        """
        self.refresh_reading()
        return self.register(JD_AIR_QUALITY_INDEX_REG_AQI_INDEX).value(self.missing_aqi_index_value)

    @property
    def aqi_index_error(self) -> Optional[float]:
        """
        (Optional) Error on the AQI measure., _: AQI
        """
        return self.register(JD_AIR_QUALITY_INDEX_REG_AQI_INDEX_ERROR).value()

    @property
    def min_aqi_index(self) -> Optional[float]:
        """
        Minimum AQI reading, representing a good air quality. Typically 0., _: AQI
        """
        return self.register(JD_AIR_QUALITY_INDEX_REG_MIN_AQI_INDEX).value()

    @property
    def max_aqi_index(self) -> Optional[float]:
        """
        Maximum AQI reading, representing a very poor air quality., _: AQI
        """
        return self.register(JD_AIR_QUALITY_INDEX_REG_MAX_AQI_INDEX).value()

    



# Autogenerated constants for Light bulb service
from jacdac.constants import *
from jacdac.system.constants import *
JD_SERVICE_CLASS_LIGHT_BULB = const(0x1cab054c)
JD_LIGHT_BULB_REG_BRIGHTNESS = const(JD_REG_INTENSITY)
JD_LIGHT_BULB_REG_DIMMABLE = const(0x180)
JD_LIGHT_BULB_PACK_FORMATS = {
    JD_LIGHT_BULB_REG_BRIGHTNESS: "u0.16",
    JD_LIGHT_BULB_REG_DIMMABLE: "u8"
}



# Autogenerated file. Do not edit.
from jacdac.bus import Bus, SensorClient
from .constants import *
from typing import Optional


class WindSpeedClient(SensorClient):
    """
    A sensor that measures wind speed.
    Implements a client for the `Wind speed <https://microsoft.github.io/jacdac-docs/services/windspeed>`_ service.

    """

    def __init__(self, bus: Bus, role: str, *, missing_wind_speed_value: float = None) -> None:
        super().__init__(bus, JD_SERVICE_CLASS_WIND_SPEED, JD_WIND_SPEED_PACK_FORMATS, role, preferred_interval = 60000)
        self.missing_wind_speed_value = missing_wind_speed_value

    @property
    def wind_speed(self) -> Optional[float]:
        """
        The velocity of the wind., _: m/s
        """
        self.refresh_reading()
        return self.register(JD_WIND_SPEED_REG_WIND_SPEED).value(self.missing_wind_speed_value)

    @property
    def wind_speed_error(self) -> Optional[float]:
        """
        (Optional) Error on the reading, _: m/s
        """
        return self.register(JD_WIND_SPEED_REG_WIND_SPEED_ERROR).value()

    @property
    def max_wind_speed(self) -> Optional[float]:
        """
        (Optional) Maximum speed that can be measured by the sensor., _: m/s
        """
        return self.register(JD_WIND_SPEED_REG_MAX_WIND_SPEED).value()

    



# Autogenerated file. Do not edit.
from jacdac.bus import Bus, SensorClient
from .constants import *
from typing import Optional


class RotaryEncoderClient(SensorClient):
    """
    An incremental rotary encoder - converts angular motion of a shaft to digital signal.
    Implements a client for the `Rotary encoder <https://microsoft.github.io/jacdac-docs/services/rotaryencoder>`_ service.

    """

    def __init__(self, bus: Bus, role: str, *, missing_position_value: int = None) -> None:
        super().__init__(bus, JD_SERVICE_CLASS_ROTARY_ENCODER, JD_ROTARY_ENCODER_PACK_FORMATS, role)
        self.missing_position_value = missing_position_value

    @property
    def position(self) -> Optional[int]:
        """
        Upon device reset starts at `0` (regardless of the shaft position).
        Increases by `1` for a clockwise "click", by `-1` for counter-clockwise., _: #
        """
        self.refresh_reading()
        return self.register(JD_ROTARY_ENCODER_REG_POSITION).value(self.missing_position_value)

    @property
    def clicks_per_turn(self) -> Optional[int]:
        """
        This specifies by how much `position` changes when the crank does 360 degree turn. Typically 12 or 24., _: #
        """
        return self.register(JD_ROTARY_ENCODER_REG_CLICKS_PER_TURN).value()

    @property
    def clicker(self) -> Optional[bool]:
        """
        (Optional) The encoder is combined with a clicker. If this is the case, the clicker button service
        should follow this service in the service list of the device., 
        """
        return self.register(JD_ROTARY_ENCODER_REG_CLICKER).bool_value()

    



# Autogenerated file. Do not edit.
from jacdac.bus import Bus, SensorClient
from .constants import *
from typing import Optional


class SoundSpectrumClient(SensorClient):
    """
    A microphone that analyzes the sound specturm
    Implements a client for the `Sound Spectrum <https://microsoft.github.io/jacdac-docs/services/soundspectrum>`_ service.

    """

    def __init__(self, bus: Bus, role: str, *, missing_frequency_bins_value: bytes = None) -> None:
        super().__init__(bus, JD_SERVICE_CLASS_SOUND_SPECTRUM, JD_SOUND_SPECTRUM_PACK_FORMATS, role)
        self.missing_frequency_bins_value = missing_frequency_bins_value

    @property
    def frequency_bins(self) -> Optional[bytes]:
        """
        The computed frequency data., 
        """
        self.refresh_reading()
        return self.register(JD_SOUND_SPECTRUM_REG_FREQUENCY_BINS).value(self.missing_frequency_bins_value)

    @property
    def enabled(self) -> Optional[bool]:
        """
        Turns on/off the micropohone., 
        """
        return self.register(JD_SOUND_SPECTRUM_REG_ENABLED).bool_value()

    @enabled.setter
    def enabled(self, value: bool) -> None:
        self.register(JD_SOUND_SPECTRUM_REG_ENABLED).set_values(value)


    @property
    def fft_pow2_size(self) -> Optional[int]:
        """
        The power of 2 used as the size of the FFT to be used to determine the frequency domain., 
        """
        return self.register(JD_SOUND_SPECTRUM_REG_FFT_POW2_SIZE).value()

    @fft_pow2_size.setter
    def fft_pow2_size(self, value: int) -> None:
        self.register(JD_SOUND_SPECTRUM_REG_FFT_POW2_SIZE).set_values(value)


    @property
    def min_decibels(self) -> Optional[int]:
        """
        The minimum power value in the scaling range for the FFT analysis data, _: dB
        """
        return self.register(JD_SOUND_SPECTRUM_REG_MIN_DECIBELS).value()

    @min_decibels.setter
    def min_decibels(self, value: int) -> None:
        self.register(JD_SOUND_SPECTRUM_REG_MIN_DECIBELS).set_values(value)


    @property
    def max_decibels(self) -> Optional[int]:
        """
        The maximum power value in the scaling range for the FFT analysis data, _: dB
        """
        return self.register(JD_SOUND_SPECTRUM_REG_MAX_DECIBELS).value()

    @max_decibels.setter
    def max_decibels(self, value: int) -> None:
        self.register(JD_SOUND_SPECTRUM_REG_MAX_DECIBELS).set_values(value)


    @property
    def smoothing_time_constant(self) -> Optional[float]:
        """
        The averaging constant with the last analysis frame.
        If `0` is set, there is no averaging done, whereas a value of `1` means "overlap the previous and current buffer quite a lot while computing the value"., _: /
        """
        return self.register(JD_SOUND_SPECTRUM_REG_SMOOTHING_TIME_CONSTANT).float_value(100)

    @smoothing_time_constant.setter
    def smoothing_time_constant(self, value: float) -> None:
        self.register(JD_SOUND_SPECTRUM_REG_SMOOTHING_TIME_CONSTANT).set_values(value / 100)


    



# Autogenerated constants for Model Runner service
from enum import IntEnum
from jacdac.constants import *
from jacdac.system.constants import *
JD_SERVICE_CLASS_MODEL_RUNNER = const(0x140f9a78)


class ModelRunnerModelFormat(IntEnum):
    TFLITE = const(0x334c4654)
    ML4F = const(0x30470f62)
    EDGE_IMPULSE_COMPILED = const(0x30564945)


JD_MODEL_RUNNER_CMD_SET_MODEL = const(0x80)
JD_MODEL_RUNNER_CMD_PREDICT = const(0x81)
JD_MODEL_RUNNER_REG_AUTO_INVOKE_EVERY = const(0x80)
JD_MODEL_RUNNER_REG_OUTPUTS = const(JD_REG_READING)
JD_MODEL_RUNNER_REG_INPUT_SHAPE = const(0x180)
JD_MODEL_RUNNER_REG_OUTPUT_SHAPE = const(0x181)
JD_MODEL_RUNNER_REG_LAST_RUN_TIME = const(0x182)
JD_MODEL_RUNNER_REG_ALLOCATED_ARENA_SIZE = const(0x183)
JD_MODEL_RUNNER_REG_MODEL_SIZE = const(0x184)
JD_MODEL_RUNNER_REG_LAST_ERROR = const(0x185)
JD_MODEL_RUNNER_REG_FORMAT = const(0x186)
JD_MODEL_RUNNER_REG_FORMAT_VERSION = const(0x187)
JD_MODEL_RUNNER_REG_PARALLEL = const(0x188)
JD_MODEL_RUNNER_PACK_FORMATS = {
    JD_MODEL_RUNNER_CMD_SET_MODEL: "u32",
    JD_MODEL_RUNNER_CMD_PREDICT: "b[12]",
    JD_MODEL_RUNNER_REG_AUTO_INVOKE_EVERY: "u16",
    JD_MODEL_RUNNER_REG_OUTPUTS: "r: f32",
    JD_MODEL_RUNNER_REG_INPUT_SHAPE: "r: u16",
    JD_MODEL_RUNNER_REG_OUTPUT_SHAPE: "r: u16",
    JD_MODEL_RUNNER_REG_LAST_RUN_TIME: "u32",
    JD_MODEL_RUNNER_REG_ALLOCATED_ARENA_SIZE: "u32",
    JD_MODEL_RUNNER_REG_MODEL_SIZE: "u32",
    JD_MODEL_RUNNER_REG_LAST_ERROR: "s",
    JD_MODEL_RUNNER_REG_FORMAT: "u32",
    JD_MODEL_RUNNER_REG_FORMAT_VERSION: "u32",
    JD_MODEL_RUNNER_REG_PARALLEL: "u8"
}



# Autogenerated constants for Total Volatile organic compound service
from jacdac.constants import *
from jacdac.system.constants import *
JD_SERVICE_CLASS_TVOC = const(0x12a5b597)
JD_TVOC_REG_TVOC = const(JD_REG_READING)
JD_TVOC_REG_TVOC_ERROR = const(JD_REG_READING_ERROR)
JD_TVOC_REG_MIN_TVOC = const(JD_REG_MIN_READING)
JD_TVOC_REG_MAX_TVOC = const(JD_REG_MAX_READING)
JD_TVOC_PACK_FORMATS = {
    JD_TVOC_REG_TVOC: "u22.10",
    JD_TVOC_REG_TVOC_ERROR: "u22.10",
    JD_TVOC_REG_MIN_TVOC: "u22.10",
    JD_TVOC_REG_MAX_TVOC: "u22.10"
}



# Autogenerated constants for Gamepad service
from enum import IntEnum
from jacdac.constants import *
from jacdac.system.constants import *
JD_SERVICE_CLASS_GAMEPAD = const(0x108f7456)


class GamepadButtons(IntEnum):
    LEFT = const(0x1)
    UP = const(0x2)
    RIGHT = const(0x4)
    DOWN = const(0x8)
    A = const(0x10)
    B = const(0x20)
    MENU = const(0x40)
    SELECT = const(0x80)
    RESET = const(0x100)
    EXIT = const(0x200)
    X = const(0x400)
    Y = const(0x800)


class GamepadVariant(IntEnum):
    THUMB = const(0x1)
    ARCADE_BALL = const(0x2)
    ARCADE_STICK = const(0x3)
    GAMEPAD = const(0x4)


JD_GAMEPAD_REG_DIRECTION = const(JD_REG_READING)
JD_GAMEPAD_REG_VARIANT = const(JD_REG_VARIANT)
JD_GAMEPAD_REG_BUTTONS_AVAILABLE = const(0x180)
JD_GAMEPAD_EV_BUTTONS_CHANGED = const(JD_EV_CHANGE)
JD_GAMEPAD_PACK_FORMATS = {
    JD_GAMEPAD_REG_DIRECTION: "u32 i1.15 i1.15",
    JD_GAMEPAD_REG_VARIANT: "u8",
    JD_GAMEPAD_REG_BUTTONS_AVAILABLE: "u32",
    JD_GAMEPAD_EV_BUTTONS_CHANGED: "u32"
}



# Autogenerated file. Do not edit.
from jacdac.bus import Bus, Client
from .constants import *
from typing import Optional


class BuzzerClient(Client):
    """
    A simple buzzer.
    Implements a client for the `Buzzer <https://microsoft.github.io/jacdac-docs/services/buzzer>`_ service.

    """

    def __init__(self, bus: Bus, role: str) -> None:
        super().__init__(bus, JD_SERVICE_CLASS_BUZZER, JD_BUZZER_PACK_FORMATS, role)


    @property
    def volume(self) -> Optional[float]:
        """
        The volume (duty cycle) of the buzzer., _: /
        """
        return self.register(JD_BUZZER_REG_VOLUME).float_value(100)

    @volume.setter
    def volume(self, value: float) -> None:
        self.register(JD_BUZZER_REG_VOLUME).set_values(value / 100)



    def play_tone(self, period: int, duty: int, duration: int) -> None:
        """
        Play a PWM tone with given period and duty for given duration.
        The duty is scaled down with `volume` register.
        To play tone at frequency `F` Hz and volume `V` (in `0..1`) you will want
        to send `P = 1000000 / F` and `D = P * V / 2`.
        """
        self.send_cmd_packed(JD_BUZZER_CMD_PLAY_TONE, period, duty, duration)

    def play_note(self, frequency: int, volume: float, duration: int) -> None:
        """
        Play a note at the given frequency and volume.
        """
        # TODO: implement client command
        raise RuntimeError("client command not implemented")
    



# Autogenerated constants for Sound level service
from jacdac.constants import *
from jacdac.system.constants import *
JD_SERVICE_CLASS_SOUND_LEVEL = const(0x14ad1a5d)
JD_SOUND_LEVEL_REG_SOUND_LEVEL = const(JD_REG_READING)
JD_SOUND_LEVEL_REG_ENABLED = const(JD_REG_INTENSITY)
JD_SOUND_LEVEL_PACK_FORMATS = {
    JD_SOUND_LEVEL_REG_SOUND_LEVEL: "u0.16",
    JD_SOUND_LEVEL_REG_ENABLED: "u8"
}



# Autogenerated file. Do not edit.
from jacdac.bus import Bus, SensorClient
from .constants import *
from typing import Optional


class TvocClient(SensorClient):
    """
    Measures equivalent Total Volatile Organic Compound levels.
    Implements a client for the `Total Volatile organic compound <https://microsoft.github.io/jacdac-docs/services/tvoc>`_ service.

    """

    def __init__(self, bus: Bus, role: str, *, missing_TVOC_value: float = None) -> None:
        super().__init__(bus, JD_SERVICE_CLASS_TVOC, JD_TVOC_PACK_FORMATS, role)
        self.missing_TVOC_value = missing_TVOC_value

    @property
    def TVOC(self) -> Optional[float]:
        """
        Total volatile organic compound readings in parts per billion., _: ppb
        """
        self.refresh_reading()
        return self.register(JD_TVOC_REG_TVOC).value(self.missing_TVOC_value)

    @property
    def TVOC_error(self) -> Optional[float]:
        """
        (Optional) Error on the reading data, _: ppb
        """
        return self.register(JD_TVOC_REG_TVOC_ERROR).value()

    @property
    def min_TVOC(self) -> Optional[float]:
        """
        Minimum measurable value, _: ppb
        """
        return self.register(JD_TVOC_REG_MIN_TVOC).value()

    @property
    def max_TVOC(self) -> Optional[float]:
        """
        Minimum measurable value., _: ppb
        """
        return self.register(JD_TVOC_REG_MAX_TVOC).value()

    



# Autogenerated file. Do not edit.
from jacdac.bus import Bus, Client
from .constants import *
from typing import Optional


class CharacterScreenClient(Client):
    """
    A screen that displays characters, typically a LCD/OLED character screen.
    Implements a client for the `Character Screen <https://microsoft.github.io/jacdac-docs/services/characterscreen>`_ service.

    """

    def __init__(self, bus: Bus, role: str) -> None:
        super().__init__(bus, JD_SERVICE_CLASS_CHARACTER_SCREEN, JD_CHARACTER_SCREEN_PACK_FORMATS, role)


    @property
    def message(self) -> Optional[str]:
        """
        Text to show. Use `\n` to break lines., 
        """
        return self.register(JD_CHARACTER_SCREEN_REG_MESSAGE).value()

    @message.setter
    def message(self, value: str) -> None:
        self.register(JD_CHARACTER_SCREEN_REG_MESSAGE).set_values(value)


    @property
    def brightness(self) -> Optional[float]:
        """
        (Optional) Brightness of the screen. `0` means off., _: /
        """
        return self.register(JD_CHARACTER_SCREEN_REG_BRIGHTNESS).float_value(100)

    @brightness.setter
    def brightness(self, value: float) -> None:
        self.register(JD_CHARACTER_SCREEN_REG_BRIGHTNESS).set_values(value / 100)


    @property
    def variant(self) -> Optional[CharacterScreenVariant]:
        """
        (Optional) Describes the type of character LED screen., 
        """
        return self.register(JD_CHARACTER_SCREEN_REG_VARIANT).value()

    @property
    def text_direction(self) -> Optional[CharacterScreenTextDirection]:
        """
        (Optional) Specifies the RTL or LTR direction of the text., 
        """
        return self.register(JD_CHARACTER_SCREEN_REG_TEXT_DIRECTION).value()

    @text_direction.setter
    def text_direction(self, value: CharacterScreenTextDirection) -> None:
        self.register(JD_CHARACTER_SCREEN_REG_TEXT_DIRECTION).set_values(value)


    @property
    def rows(self) -> Optional[int]:
        """
        Gets the number of rows., _: #
        """
        return self.register(JD_CHARACTER_SCREEN_REG_ROWS).value()

    @property
    def columns(self) -> Optional[int]:
        """
        Gets the number of columns., _: #
        """
        return self.register(JD_CHARACTER_SCREEN_REG_COLUMNS).value()

    



# Autogenerated constants for Relay service
from enum import IntEnum
from jacdac.constants import *
from jacdac.system.constants import *
JD_SERVICE_CLASS_RELAY = const(0x183fe656)


class RelayVariant(IntEnum):
    ELECTROMECHANICAL = const(0x1)
    SOLID_STATE = const(0x2)
    REED = const(0x3)


JD_RELAY_REG_ACTIVE = const(JD_REG_INTENSITY)
JD_RELAY_REG_VARIANT = const(JD_REG_VARIANT)
JD_RELAY_REG_MAX_SWITCHING_CURRENT = const(0x180)
JD_RELAY_PACK_FORMATS = {
    JD_RELAY_REG_ACTIVE: "u8",
    JD_RELAY_REG_VARIANT: "u8",
    JD_RELAY_REG_MAX_SWITCHING_CURRENT: "u32"
}



# Autogenerated constants for Barcode reader service
from enum import IntEnum
from jacdac.constants import *
from jacdac.system.constants import *
JD_SERVICE_CLASS_BARCODE_READER = const(0x1c739e6c)


class BarcodeReaderFormat(IntEnum):
    AZTEC = const(0x1)
    CODE128 = const(0x2)
    CODE39 = const(0x3)
    CODE93 = const(0x4)
    CODABAR = const(0x5)
    DATA_MATRIX = const(0x6)
    EAN13 = const(0x8)
    EAN8 = const(0x9)
    ITF = const(0xa)
    PDF417 = const(0xb)
    QR_CODE = const(0xc)
    UPC_A = const(0xd)
    UPC_E = const(0xe)


JD_BARCODE_READER_REG_ENABLED = const(JD_REG_INTENSITY)
JD_BARCODE_READER_REG_FORMATS = const(0x180)
JD_BARCODE_READER_EV_DETECT = const(JD_EV_ACTIVE)
JD_BARCODE_READER_PACK_FORMATS = {
    JD_BARCODE_READER_REG_ENABLED: "u8",
    JD_BARCODE_READER_REG_FORMATS: "r: u8",
    JD_BARCODE_READER_EV_DETECT: "u8 s"
}



# Autogenerated constants for LED Single service
from enum import IntEnum
from jacdac.constants import *
from jacdac.system.constants import *
JD_SERVICE_CLASS_LED_SINGLE = const(0x1e3048f8)


class LedSingleVariant(IntEnum):
    THROUGH_HOLE = const(0x1)
    SMD = const(0x2)
    POWER = const(0x3)
    BEAD = const(0x4)


JD_LED_SINGLE_CMD_ANIMATE = const(0x80)
JD_LED_SINGLE_REG_COLOR = const(0x180)
JD_LED_SINGLE_REG_MAX_POWER = const(JD_REG_MAX_POWER)
JD_LED_SINGLE_REG_LED_COUNT = const(0x183)
JD_LED_SINGLE_REG_WAVE_LENGTH = const(0x181)
JD_LED_SINGLE_REG_LUMINOUS_INTENSITY = const(0x182)
JD_LED_SINGLE_REG_VARIANT = const(JD_REG_VARIANT)
JD_LED_SINGLE_PACK_FORMATS = {
    JD_LED_SINGLE_CMD_ANIMATE: "u8 u8 u8 u8",
    JD_LED_SINGLE_REG_COLOR: "u8 u8 u8",
    JD_LED_SINGLE_REG_MAX_POWER: "u16",
    JD_LED_SINGLE_REG_LED_COUNT: "u16",
    JD_LED_SINGLE_REG_WAVE_LENGTH: "u16",
    JD_LED_SINGLE_REG_LUMINOUS_INTENSITY: "u16",
    JD_LED_SINGLE_REG_VARIANT: "u8"
}



import binascii
import time
from typing import Union

logging = True
loggingv = False
_hex = "0123456789abcdef"


def now():
    return int(time.monotonic() * 1000)


def log(msg: str, *args: object):
    if logging:
        if len(args):
            msg = msg.format(*args)
        print(msg)


def logv(msg: str, *args: object):
    if loggingv:
        if len(args):
            msg = msg.format(*args)
        print(msg)


def hex_num(n: int, len: int = 8):
    r = "0x"
    for i in range(len):
        r += _hex[(n >> ((len - 1 - i) * 4)) & 0xf]
    return r


def buf2hex(buf: bytes):
    return binascii.hexlify(buf).decode()
    # r = ""
    # # is this quadartic?
    # for b in buf:
    #     r += _hex[b >> 4] + _hex[b & 0xf]
    # return r


def hex2buf(s: str):
    return binascii.unhexlify(s)
    # r = bytearray(len(s) >> 1)
    # for idx in range(0, len(s), 2):
    #     r[idx >> 1] = (_hex.index(s[idx].lower()) <<
    #                    4) | _hex.index(s[idx+1].lower())
    # return r


def u16(buf: bytes, off: int):
    return buf[off] | (buf[off+1] << 8)


def set_u16(buf: bytearray, off: int, val: int):
    buf[off] = val & 0xff
    buf[off + 1] = val >> 8


def u32(buf: bytes, off: int):
    return buf[off] | (buf[off+1] << 8) | (buf[off+2] << 16) | (buf[off+3] << 24)


def hash(buf: bytes, bits: int = 30):
    # return busio.JACDAC.__dict__["hash"](buf, bits)
    if bits < 1:
        return 0
    h = fnv1(buf)
    if bits >= 32:
        return h >> 0
    else:
        return ((h ^ (h >> bits)) & ((1 << bits) - 1))


def fnv1(data: bytes):
    h = 0x811c9dc5
    for i in range(len(data)):
        h = ((h * 0x1000193) & 0xffff_ffff) ^ data[i]
    return h


def short_id(longid: Union[bytes, str]):
    if isinstance(longid, str):
        longid = hex2buf(longid)
    h = hash(longid)
    return (
        chr(0x41 + h % 26) +
        chr(0x41 + (h // 26) % 26) +
        chr(0x30 + (h // (26 * 26)) % 10) +
        chr(0x30 + (h // (26 * 26 * 10)) % 10)
    )


def crc16(buf: bytes, start: int = 0, end: int = None):
    if end is None:
        end = len(buf)
    crc = 0xffff
    while start < end:
        data = buf[start]
        start += 1
        x = (crc >> 8) ^ data
        x ^= x >> 4
        crc = ((crc << 8) ^ (x << 12) ^ (x << 5) ^ x) & 0xffff
    return crc



# Autogenerated constants for Dual Motors service
from jacdac.constants import *
from jacdac.system.constants import *
JD_SERVICE_CLASS_DUAL_MOTORS = const(0x1529d537)
JD_DUAL_MOTORS_REG_SPEED = const(JD_REG_VALUE)
JD_DUAL_MOTORS_REG_ENABLED = const(JD_REG_INTENSITY)
JD_DUAL_MOTORS_REG_LOAD_TORQUE = const(0x180)
JD_DUAL_MOTORS_REG_LOAD_ROTATION_SPEED = const(0x181)
JD_DUAL_MOTORS_REG_REVERSIBLE = const(0x182)
JD_DUAL_MOTORS_PACK_FORMATS = {
    JD_DUAL_MOTORS_REG_SPEED: "i1.15 i1.15",
    JD_DUAL_MOTORS_REG_ENABLED: "u8",
    JD_DUAL_MOTORS_REG_LOAD_TORQUE: "u16.16",
    JD_DUAL_MOTORS_REG_LOAD_ROTATION_SPEED: "u16.16",
    JD_DUAL_MOTORS_REG_REVERSIBLE: "u8"
}



# Autogenerated file. Do not edit.
from jacdac.bus import Bus, Client
from .constants import *



class VibrationMotorClient(Client):
    """
    A vibration motor.
    Implements a client for the `Vibration motor <https://microsoft.github.io/jacdac-docs/services/vibrationmotor>`_ service.

    """

    def __init__(self, bus: Bus, role: str) -> None:
        super().__init__(bus, JD_SERVICE_CLASS_VIBRATION_MOTOR, JD_VIBRATION_MOTOR_PACK_FORMATS, role)


    



# Autogenerated file. Do not edit.
from jacdac.bus import Bus, SensorClient, EventHandlerFn, UnsubscribeFn
from .constants import *
from typing import Optional


class WifiClient(SensorClient):
    """
    Discovery and connection to WiFi networks. Separate TCP service can be used for data transfer.
    Implements a client for the `WIFI <https://microsoft.github.io/jacdac-docs/services/wifi>`_ service.

    """

    def __init__(self, bus: Bus, role: str, *, missing_rssi_value: int = None) -> None:
        super().__init__(bus, JD_SERVICE_CLASS_WIFI, JD_WIFI_PACK_FORMATS, role)
        self.missing_rssi_value = missing_rssi_value

    @property
    def rssi(self) -> Optional[int]:
        """
        Current signal strength. Returns -128 when not connected., _: dB
        """
        self.refresh_reading()
        return self.register(JD_WIFI_REG_RSSI).value(self.missing_rssi_value)

    @property
    def enabled(self) -> Optional[bool]:
        """
        Determines whether the WiFi radio is enabled. It starts enabled upon reset., 
        """
        return self.register(JD_WIFI_REG_ENABLED).bool_value()

    @enabled.setter
    def enabled(self, value: bool) -> None:
        self.register(JD_WIFI_REG_ENABLED).set_values(value)


    @property
    def ip_address(self) -> Optional[bytes]:
        """
        0, 4 or 16 byte buffer with the IPv4 or IPv6 address assigned to device if any., 
        """
        return self.register(JD_WIFI_REG_IP_ADDRESS).value()

    @property
    def eui_48(self) -> Optional[bytes]:
        """
        The 6-byte MAC address of the device. If a device does MAC address randomization it will have to "restart"., 
        """
        return self.register(JD_WIFI_REG_EUI_48).value()

    @property
    def ssid(self) -> Optional[str]:
        """
        SSID of the access-point to which device is currently connected.
        Empty string if not connected., 
        """
        return self.register(JD_WIFI_REG_SSID).value()

    def on_got_ip(self, handler: EventHandlerFn) -> UnsubscribeFn:
        """
        Emitted upon successful join and IP address assignment.
        """
        return self.on_event(JD_WIFI_EV_GOT_IP, handler)

    def on_lost_ip(self, handler: EventHandlerFn) -> UnsubscribeFn:
        """
        Emitted when disconnected from network.
        """
        return self.on_event(JD_WIFI_EV_LOST_IP, handler)

    def on_scan_complete(self, handler: EventHandlerFn) -> UnsubscribeFn:
        """
        A WiFi network scan has completed. Results can be read with the `last_scan_results` command.
        The event indicates how many networks where found, and how many are considered
        as candidates for connection.
        """
        return self.on_event(JD_WIFI_EV_SCAN_COMPLETE, handler)

    def on_networks_changed(self, handler: EventHandlerFn) -> UnsubscribeFn:
        """
        Emitted whenever the list of known networks is updated.
        """
        return self.on_event(JD_WIFI_EV_NETWORKS_CHANGED, handler)

    def on_connection_failed(self, handler: EventHandlerFn) -> UnsubscribeFn:
        """
        Emitted when when a network was detected in scan, the device tried to connect to it
        and failed.
        This may be because of wrong password or other random failure.
        """
        return self.on_event(JD_WIFI_EV_CONNECTION_FAILED, handler)


    def add_network(self, ssid: str, password: str) -> None:
        """
        Automatically connect to named network if available. Also set password if network is not open.
        """
        self.send_cmd_packed(JD_WIFI_CMD_ADD_NETWORK, ssid, password)

    def reconnect(self, ) -> None:
        """
        Enable the WiFi (if disabled), initiate a scan, wait for results, disconnect from current WiFi network if any,
        and then reconnect (using regular algorithm, see `set_network_priority`).
        """
        self.send_cmd_packed(JD_WIFI_CMD_RECONNECT, )

    def forget_network(self, ssid: str) -> None:
        """
        Prevent from automatically connecting to named network in future.
        Forgetting a network resets its priority to `0`.
        """
        self.send_cmd_packed(JD_WIFI_CMD_FORGET_NETWORK, ssid)

    def forget_all_networks(self, ) -> None:
        """
        Clear the list of known networks.
        """
        self.send_cmd_packed(JD_WIFI_CMD_FORGET_ALL_NETWORKS, )

    def set_network_priority(self, priority: int, ssid: str) -> None:
        """
        Set connection priority for a network.
        By default, all known networks have priority of `0`.
        """
        self.send_cmd_packed(JD_WIFI_CMD_SET_NETWORK_PRIORITY, priority, ssid)

    def scan(self, ) -> None:
        """
        Initiate search for WiFi networks. Generates `scan_complete` event.
        """
        self.send_cmd_packed(JD_WIFI_CMD_SCAN, )
    



# Autogenerated constants for HID Keyboard service
from enum import IntEnum
from jacdac.constants import *
JD_SERVICE_CLASS_HID_KEYBOARD = const(0x18b05b6a)


class HidKeyboardModifiers(IntEnum):
    NONE = const(0x0)
    LEFT_CONTROL = const(0x1)
    LEFT_SHIFT = const(0x2)
    LEFT_ALT = const(0x4)
    LEFT_GUI = const(0x8)
    RIGHT_CONTROL = const(0x10)
    RIGHT_SHIFT = const(0x20)
    RIGHT_ALT = const(0x40)
    RIGHT_GUI = const(0x80)


class HidKeyboardAction(IntEnum):
    PRESS = const(0x0)
    UP = const(0x1)
    DOWN = const(0x2)


JD_HID_KEYBOARD_CMD_KEY = const(0x80)
JD_HID_KEYBOARD_CMD_CLEAR = const(0x81)
JD_HID_KEYBOARD_PACK_FORMATS = {
    JD_HID_KEYBOARD_CMD_KEY: "r: u16 u8 u8"
}
